@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -81,28 +81,55 @@ class Document
 
 	function getName() { return $this->_name; }
 
-	function setName($newName)
-	{
-		GLOBAL $db;
+	function setName($newName) {
+		GLOBAL $db, $user, $settings;
 		
 		$queryStr = "UPDATE tblDocuments SET name = '" . $newName . "' WHERE id = ". $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Document Notification]: ".$this->_name;
+		$message = "Document '".$this->_name."' has been renamed to '".$newName."'.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\tOld Name: ".$this->_name."\r\n".
+			"\tNew Name: ".$newName."\r\n".
+			"\t  Folder: ".getFolderPathPlain($this->getFolder())."\r\n".
+			"\t Comment: ".$this->getComment()."\r\n".
+			"\t     URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
 		$this->_name = $newName;
 		return true;
 	}
 
 	function getComment() { return $this->_comment; }
 
-	function setComment($newComment)
-	{
-		GLOBAL $db;
-		
+	function setComment($newComment) {
+		GLOBAL $db, $user, $settings;
+
 		$queryStr = "UPDATE tblDocuments SET comment = '" . $newComment . "' WHERE id = ". $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Document Notification]: ".$this->_name;
+		$message = "The comment for document '".$this->_name."' has been changed.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t       Name: ".$this->_name."\r\n".
+			"\t     Folder: ".getFolderPathPlain($this->getFolder())."\r\n".
+			"\tNew Comment: ".$newComment."\r\n".
+			"\t        URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
 		$this->_comment = $newComment;
 		return true;
 	}
@@ -111,7 +138,7 @@ class Document
 
 	function setKeywords($newKeywords)
 	{
-		GLOBAL $db;
+		GLOBAL $db, $user, $settings;
 		
 		$queryStr = "UPDATE tblDocuments SET keywords = '" . $newKeywords . "' WHERE id = ". $this->_id;
 		if (!$db->getResult($queryStr))
@@ -135,7 +162,7 @@ class Document
 
 	function setFolder($newFolder)
 	{
-		GLOBAL $db;
+		GLOBAL $db, $user, $settings;
 		
 		$queryStr = "UPDATE tblDocuments SET folder = " . $newFolder->getID() . " WHERE id = ". $this->_id;
 		if (!$db->getResult($queryStr))
@@ -156,26 +183,56 @@ class Document
 		if (!$db->getResult($queryStr))
 			return false;
 
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Document Notification]: ".$this->_name;
+		$message = "Document '".$this->_name."' has been moved to a new folder.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t      Name: ".$this->_name."\r\n".
+			"\tNew Folder: ".getFolderPathPlain($this->getFolder())."\r\n".
+			"\t   Comment: ".$newComment."\r\n".
+			"\t       URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
 		return true;
 	}
 
-	function getOwner()
-	{
+	function getOwner() {
 		if (!isset($this->_owner))
 			$this->_owner = getUser($this->_ownerID);
 		return $this->_owner;
 	}
 
-	function setOwner($user)
-	{
-		GLOBAL $db;
-		
-		$queryStr = "UPDATE tblDocuments set owner = " . $user->getID() . " WHERE id = " . $this->_id;
+	function setOwner($newOwner) {
+		GLOBAL $db, $user, $settings;
+
+		$oldOwner = $this->getOwner();
+
+		$queryStr = "UPDATE tblDocuments set owner = " . $newOwner->getID() . " WHERE id = " . $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
-		
-		$this->_ownerID = $user->getID();
-		$this->_owner = $user;
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Document Notification]: ".$this->_name;
+		$message = "Ownership has changed for document '".$this->_name."'. Old owner: '".$oldOwner->getFullName()."'. New Owner: '".$newOwner->getFullName()."'";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->_name."\r\n".
+			"\t Folder: ".getFolderPathPlain($this->getFolder())."\r\n".
+			"\tComment: ".$this->_comment."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+		// Send notification to previous owner.
+		Email::toIndividual($user, $oldOwner, $subject, $message);
+
+		$this->_ownerID = $newOwner->getID();
+		$this->_owner = $newOwner;
 		return true;
 	}
 
@@ -190,31 +247,83 @@ class Document
 		return $this->_defaultAccess;
 	}
 
-	function setDefaultAccess($mode)
-	{
-		GLOBAL $db;
+	function setDefaultAccess($mode) {
+		GLOBAL $db, $user, $settings;
 		
 		$queryStr = "UPDATE tblDocuments set defaultAccess = " . $mode . " WHERE id = " . $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
-		
-		$this->_defaulAccess = $mode;
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Document Notification]: ".$this->_name;
+		$message = "Default access privileges have changed for document '".$this->_name."'.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->_name."\r\n".
+			"\t Folder: ".getFolderPathPlain($this->getFolder())."\r\n".
+			"\tComment: ".$comment."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
+		$this->_defaultAccess = $mode;
+
+		// If any of the notification subscribers no longer have read access,
+		// remove their subscription.
+		foreach ($this->_notifyList["users"] as $u) {
+			if ($this->getAccessMode($u) < M_READ) {
+				$this->removeNotify($u->getID(), true);
+			}
+		}
+		foreach ($this->_notifyList["groups"] as $g) {
+			if ($this->getGroupAccessMode($g) < M_READ) {
+				$this->removeNotify($g->getID(), false);
+			}
+		}
+
 		return true;
 	}
 
 	function inheritsAccess() { return $this->_inheritAccess; }
 
-	function setInheritAccess($inheritAccess)
-	{
-		GLOBAL $db;
+	function setInheritAccess($inheritAccess) {
+		GLOBAL $db, $user, $settings;
 		
-		$inheritAccess = ($inheritAccess) ? "1" : "0";
-		
-		$queryStr = "UPDATE tblDocuments SET inheritAccess = " . $inheritAccess . " WHERE id = " . $this->_id;
+		$queryStr = "UPDATE tblDocuments SET inheritAccess = " . ($inheritAccess ? "1" : "0") . " WHERE id = " . $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
 		
-		$this->_inheritAccess = $inheritAccess;
+		$this->_inheritAccess = ($inheritAccess ? "1" : "0");
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Document Notification]: ".$this->_name;
+		$message = "Default access privileges have changed for document '".$this->_name."'. Access is ".($inheritAccess ? "now" : "no longer")." inherited.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->_name."\r\n".
+			"\t Folder: ".getFolderPathPlain($this->getFolder())."\r\n".
+			"\tComment: ".$newComment."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
+		// If any of the notification subscribers no longer have read access,
+		// remove their subscription.
+		foreach ($this->_notifyList["users"] as $u) {
+			if ($this->getAccessMode($u) < M_READ) {
+				$this->removeNotify($u->getID(), true);
+			}
+		}
+		foreach ($this->_notifyList["groups"] as $g) {
+			if ($this->getGroupAccessMode($g) < M_READ) {
+				$this->removeNotify($g->getID(), false);
+			}
+		}
+
 		return true;
 	}
 
@@ -234,16 +343,34 @@ class Document
 			return $this->_expires;
 	}
 
-	function setExpires($expires)
-	{
-		GLOBAL $db;
+	function setExpires($expires) {
+		GLOBAL $db, $user, $settings;
 		
 		$expires = (!$expires) ? 0 : $expires;
-		
+
+		if ($expires == $this->_expires) {
+			// No change is necessary.
+			return true;
+		}
+
 		$queryStr = "UPDATE tblDocuments SET expires = " . $expires . " WHERE id = " . $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Document Notification]: ".$this->_name;
+		$message = "Document '".$this->_name."' expiry has been set to '".($expires==0 ? "never" : getReadableDate($expires))."'.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->_name."\r\n".
+			"\t Folder: ".getFolderPathPlain($this->getFolder())."\r\n".
+			"\tComment: ".$this->getComment()."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
 		$this->_expires = $expires;
 		return true;
 	}
@@ -348,8 +475,7 @@ class Document
 		return $this->_accessList[$mode];
 	}
 
-	function addAccess($mode, $userOrGroupID, $isUser)
-	{
+	function addAccess($mode, $userOrGroupID, $isUser) {
 		GLOBAL $db;
 		
 		$userOrGroup = ($isUser) ? "userID" : "groupID";
@@ -360,34 +486,51 @@ class Document
 			return false;
 		
 		unset($this->_accessList);
+
+		// Update the notify list, if necessary.
+		if ($mode == M_NONE) {
+			$this->removeNotify($userOrGroupID, $isUser);
+		}
+
 		return true;
 	}
 
-	function changeAccess($newMode, $userOrGroupID, $isUser)
-	{
+	function changeAccess($newMode, $userOrGroupID, $isUser) {
 		GLOBAL $db;
-		
+
 		$userOrGroup = ($isUser) ? "userID" : "groupID";
-		
+
 		$queryStr = "UPDATE tblACLs SET mode = " . $newMode . " WHERE targetType = ".T_DOCUMENT." AND target = " . $this->_id . " AND " . $userOrGroup . " = " . $userOrGroupID;
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
 		unset($this->_accessList);
+
+		// Update the notify list, if necessary.
+		if ($newMode == M_NONE) {
+			$this->removeNotify($userOrGroupID, $isUser);
+		}
+
 		return true;
 	}
 
-	function removeAccess($userOrGroupID, $isUser)
-	{
+	function removeAccess($userOrGroupID, $isUser) {
 		GLOBAL $db;
-		
+
 		$userOrGroup = ($isUser) ? "userID" : "groupID";
-		
+
 		$queryStr = "DELETE FROM tblACLs WHERE targetType = ".T_DOCUMENT." AND target = ".$this->_id." AND ".$userOrGroup." = " . $userOrGroupID;
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
 		unset($this->_accessList);
+
+		// Update the notify list, if necessary.
+		$mode = ($isUser ? $this->getAccessMode(getUser($userOrGroupID)) : $this->getGroupAccessMode(getGroup($userOrGroupID)));
+		if ($mode == M_NONE) {
+			$this->removeNotify($userOrGroupID, $isUser);
+		}
+
 		return true;
 	}
 
@@ -468,8 +611,38 @@ class Document
 		return $this->getDefaultAccess();
 	}
 
-	function getNotifyList()
-	{
+
+	function getGroupAccessMode($group) {
+		global $settings;
+
+		$highestPrivileged = M_NONE;
+		
+		//ACLs durchforsten
+		$foundInACL = false;
+		$accessList = $this->getAccessList();
+		if (!$accessList)
+			return false;
+		
+		foreach ($accessList["groups"] as $groupAccess) {
+			if ($groupAccess->getGroupID() == $group->getID()) {
+				$foundInACL = true;
+				if ($groupAccess->getMode() > $highestPrivileged)
+					$highestPrivileged = $groupAccess->getMode();
+				if ($highestPrivileged == M_ALL) //hher geht's nicht -> wir knnen uns die arbeit schenken
+					return $highestPrivileged;
+			}
+		}
+
+		if ($foundInACL)
+			return $highestPrivileged;
+		
+		//Standard-Berechtigung verwenden
+		return $this->getDefaultAccess();
+
+	}
+
+
+	function getNotifyList() {
 		if (!isset($this->_notifyList))
 		{
 			GLOBAL $db;
@@ -491,32 +664,222 @@ class Document
 		return $this->_notifyList;
 	}
 
-	function addNotify($userOrGroupID, $isUser)
-	{
-		GLOBAL $db;
-		
-		$userOrGroup = ($isUser) ? "userID" : "groupID";
-		
+	function addNotify($userOrGroupID, $isUser) {
+
+		// Return values:
+		// -1: Invalid User/Group ID.
+		// -2: Target User / Group does not have read access.
+		// -3: User is already subscribed.
+		// -4: Database / internal error.
+		//  0: Update successful.
+
+		global $db, $settings, $user;
+
+		$userOrGroup = ($isUser ? "userID" : "groupID");
+
+		//
+		// Verify that user / group exists.
+		//
+		$obj = ($isUser ? getUser($userOrGroupID) : getGroup($userOrGroupID));
+		if (!is_object($obj)) {
+			return -1;
+		}
+
+		//
+		// Verify that the requesting user has permission to add the target to
+		// the notification system.
+		//
+		if ($user->getID() == $settings->_guestID) {
+			return -2;
+		}
+		if (!$user->isAdmin()) {
+			if ($isUser) {
+				if ($user->getID() != $obj->getID()) {
+					return -2;
+				}
+			}
+			else {
+				if (!$obj->isMember($user)) {
+					return -2;
+				}
+			}
+		}
+
+		//
+		// Verify that target user / group has read access to the document.
+		//
+		if ($isUser) {
+			// Users are straightforward to check.
+			if ($this->getAccessMode($obj) < M_READ) {
+				return -2;
+			}
+		}
+		else {
+			// Groups are a little more complex.
+			if ($this->getDefaultAccess() >= M_READ) {
+				// If the default access is at least READ-ONLY, then just make sure
+				// that the current group has not been explicitly excluded.
+				$acl = $this->getAccessList(M_NONE, O_EQ);
+				$found = false;
+				foreach ($acl["groups"] as $group) {
+					if ($group->getGroupID() == $userOrGroupID) {
+						$found = true;
+						break;
+					}
+				}
+				if ($found) {
+					return -2;
+				}
+			}
+			else {
+				// The default access is restricted. Make sure that the group has
+				// been explicitly allocated access to the document.
+				$acl = $this->getAccessList(M_READ, O_GTEQ);
+				if (is_bool($acl)) {
+					return -4;
+				}
+				$found = false;
+				foreach ($acl["groups"] as $group) {
+					if ($group->getGroupID() == $userOrGroupID) {
+						$found = true;
+						break;
+					}
+				}
+				if (!$found) {
+					return -2;
+				}
+			}
+		}
+		//
+		// Check to see if user/group is already on the list.
+		//
+		$queryStr = "SELECT * FROM `tblNotify` WHERE `tblNotify`.`target` = '".$this->_id."' ".
+			"AND `tblNotify`.`targetType` = '".T_DOCUMENT."' ".
+			"AND `tblNotify`.`".$userOrGroup."` = '".$userOrGroupID."'";
+		$resArr = $db->getResultArray($queryStr);
+		if (is_bool($resArr)) {
+			return -4;
+		}
+		if (count($resArr)>0) {
+			return -3;
+		}
+
 		$queryStr = "INSERT INTO tblNotify (target, targetType, " . $userOrGroup . ") VALUES (" . $this->_id . ", " . T_DOCUMENT . ", " . $userOrGroupID . ")";
 		if (!$db->getResult($queryStr))
-			return false;
-		
+			return -4;
+
+		// Email user / group, informing them of subscription.
+		$path="";
+		$folder = $this->getFolder();
+		$folderPath = $folder->getPath();
+		for ($i = 0; $i  < count($folderPath); $i++) {
+			$path .= $folderPath[$i]->getName();
+			if ($i +1 < count($folderPath))
+				$path .= " / ";
+		}
+		$subject = "[Document Notification]: ".$this->getName();
+		$message = ($isUser ? "You have" : "The group '".$obj->getName()."' has")." been added to the event notification system for the following document: ";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->getName()."\r\n".
+			"\t Folder: ".$path."\r\n".
+			"\tComment: ".$this->getComment()."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_id."\r\n";
+
+		if ($isUser) {
+			Email::toIndividual($user, $obj, $subject, $message);
+		}
+		else {
+			Email::toGroup($user, $obj, $subject, $message);
+		}
+
 		unset($this->_notifyList);
-		return true;
+		return 0;
 	}
 
-	function removeNotify($userOrGroupID, $isUser)
-	{
-		GLOBAL $db;
+	function removeNotify($userOrGroupID, $isUser) {
+
+		// Return values:
+		// -1: Invalid User/Group ID.
+		// -3: User is not subscribed. No action taken.
+		// -4: Database / internal error.
+		//  0: Update successful.
+
+		GLOBAL $db, $settings, $user;
 		
+		//
+		// Verify that user / group exists.
+		//
+		$obj = ($isUser ? getUser($userOrGroupID) : getGroup($userOrGroupID));
+		if (!is_object($obj)) {
+			return -1;
+		}
+
 		$userOrGroup = ($isUser) ? "userID" : "groupID";
-		
+
+		//
+		// Verify that the requesting user has permission to add the target to
+		// the notification system.
+		//
+		if ($user->getID() == $settings->_guestID) {
+			return -2;
+		}
+		if (!$user->isAdmin()) {
+			if ($isUser) {
+				if ($user->getID() != $obj->getID()) {
+					return -2;
+				}
+			}
+			else {
+				if (!$obj->isMember($user)) {
+					return -2;
+				}
+			}
+		}
+
+		//
+		// Check to see if the target is in the database.
+		//
+		$queryStr = "SELECT * FROM `tblNotify` WHERE `tblNotify`.`target` = '".$this->_id."' ".
+			"AND `tblNotify`.`targetType` = '".T_DOCUMENT."' ".
+			"AND `tblNotify`.`".$userOrGroup."` = '".$userOrGroupID."'";
+		$resArr = $db->getResultArray($queryStr);
+		if (is_bool($resArr)) {
+			return -4;
+		}
+		if (count($resArr)==0) {
+			return -3;
+		}
+
 		$queryStr = "DELETE FROM tblNotify WHERE target = " . $this->_id . " AND targetType = " . T_DOCUMENT . " AND " . $userOrGroup . " = " . $userOrGroupID;
 		if (!$db->getResult($queryStr))
-			return false;
+			return -4;
 		
+		// Email user / group, informing them of subscription change.
+		$path="";
+		$folder = $this->getFolder();
+		$folderPath = $folder->getPath();
+		for ($i = 0; $i  < count($folderPath); $i++) {
+			$path .= $folderPath[$i]->getName();
+			if ($i +1 < count($folderPath))
+				$path .= " / ";
+		}
+		$subject = "[Document Notification]: ".$this->getName();
+		$message = ($isUser ? "You have" : "The group '".$obj->getName()."' has")." been removed from the event notification system for the following document: ";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->getName()."\r\n".
+			"\t Folder: ".$path."\r\n".
+			"\tComment: ".$this->getComment()."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_id."\r\n";
+
+		if ($isUser) {
+			Email::toIndividual($user, $obj, $subject, $message);
+		}
+		else {
+			Email::toGroup($user, $obj, $subject, $message);
+		}
+
 		unset($this->_notifyList);
-		return true;
+		return 0;
 	}
 
 
@@ -556,6 +919,8 @@ class Document
 			return false;
 		$statusID = $db->getInsertID();
 
+		$this->addNotify($user->getID(), true);
+
 		// Add reviewers into the database. Reviewers must review the document
 		// and submit comments, if appropriate. Reviewers can also recommend that
 		// a document be rejected.
@@ -612,6 +977,21 @@ class Document
 			return false;
 
 		$docResultSet->setStatus($status);
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Document Notification]: ".$this->_name;
+		$message = "Document '".$this->_name."' has been updated (Version '".$version."').";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->_name."\r\n".
+			"\t Folder: ".getFolderPathPlain($this->getFolder())."\r\n".
+			"\tComment: ".$this->getComment()."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
 		return $docResultSet;
 	}
 
@@ -724,7 +1104,7 @@ class Document
 
 	function remove()
 	{
-		GLOBAL $db, $user;
+		GLOBAL $db, $user, $settings;
 		
 		$res = $this->getContent();
 		if (is_bool($res) && !$res) return false;
@@ -737,9 +1117,6 @@ class Document
 		if (!$db->getResult($queryStr))
 			return false;
 		$queryStr = "DELETE FROM tblACLs WHERE target = " . $this->_id . " AND targetType = " . T_DOCUMENT;
-		if (!$db->getResult($queryStr))
-			return false;
-		$queryStr = "DELETE FROM tblNotify WHERE target = " . $this->_id . " AND targetType = " . T_DOCUMENT;
 		if (!$db->getResult($queryStr))
 			return false;
 		$queryStr = "DELETE FROM tblDocumentLinks WHERE document = " . $this->_id . " OR target = " . $this->_id;
@@ -762,9 +1139,18 @@ class Document
 			"has deleted the following document:";
 		$message = wordwrap ($message, 72, "\r\n");
 		$message .= "\r\n\r\n\t   Name: ".$this->getName()."\r\n".
-			"\t Folder: ".$path."\r\n";
+			"\t Folder: ".$path."\r\n".
 			"\tComment: ".$this->getComment()."\r\n";
 		Email::toIndividual($user, $this->getOwner(), $subject, $message);
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
+		// Delete the notification list.
+		$queryStr = "DELETE FROM tblNotify WHERE target = " . $this->_id . " AND targetType = " . T_DOCUMENT;
 
 		return true;
 	}
@@ -1048,6 +1434,15 @@ class DocumentContent
 			"\tVersion: ".$this->_version."\r\n".
 			"\tComment: ".$this->getComment()."\r\n";
 		Email::toList($user, $recipients, $subject, $message);
+		// Send notification to subscribers.
+		if (!isset($this->_document)) {
+			$this->_document = getDocument($this->_documentID);
+		}
+		$nl=$this->_document->getNotifyList();
+		Email::toList($user, $nl["users"], $subject, $message);
+		foreach ($nl["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
 
 		return true;
 	}
@@ -1081,7 +1476,7 @@ class DocumentContent
 	}
 
 	function setStatus($status, $comment, $updateUser = null) {
-		GLOBAL $db, $settings;
+		GLOBAL $db, $user, $settings;
 
 		// If the supplied value lies outside of the accepted range, return an
 		// error.
@@ -1102,6 +1497,25 @@ class DocumentContent
 		$res = $db->getResult($queryStr);
 		if (is_bool($res) && !$res)
 			return false;
+
+		// Send notification to subscribers.
+		if (!isset($this->_document)) {
+			$this->_document = getDocument($this->_documentID);
+		}
+		$nl=$this->_document->getNotifyList();
+		$subject = "[Document Notification]: ".$this->_document->_name;
+		$message = "Document '".$this->_document->_name."' status has been changed to '".getOverallStatusText($status)."'.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->_document->_name."\r\n".
+			"\t Folder: ".getFolderPathPlain($this->_document->getFolder())."\r\n".
+			"\tComment: ".$this->_document->getComment()."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_documentID."\r\n";
+		$uu = (is_null($updateUser) ? getUser($settings->_adminID) : $updateUser);
+		Email::toList($uu, $nl["users"], $subject, $message);
+		foreach ($nl["groups"] as $grp) {
+			Email::toGroup($uu, $grp, $subject, $message);
+		}
+
 		return true;
 	}
 
@@ -1116,10 +1530,12 @@ class DocumentContent
 			}
 			$queryStr="SELECT `tblDocumentReviewers`.*, `tblDocumentReviewLog`.`status`, ".
 				"`tblDocumentReviewLog`.`comment`, `tblDocumentReviewLog`.`date`, ".
-				"`tblDocumentReviewLog`.`userID` ".
+				"`tblDocumentReviewLog`.`userID`, `tblUsers`.`fullName`, `tblGroups`.`name` AS `groupName` ".
 				"FROM `tblDocumentReviewers` ".
 				"LEFT JOIN `tblDocumentReviewLog` USING (`reviewID`) ".
 				"LEFT JOIN `ttreviewid` on `ttreviewid`.`maxLogID` = `tblDocumentReviewLog`.`reviewLogID` ".
+				"LEFT JOIN `tblUsers` on `tblUsers`.`id` = `tblDocumentReviewers`.`required`".
+				"LEFT JOIN `tblGroups` on `tblGroups`.`id` = `tblDocumentReviewers`.`required`".
 				"WHERE `ttreviewid`.`maxLogID`=`tblDocumentReviewLog`.`reviewLogID` ".
 				"AND `tblDocumentReviewers`.`documentID` = '". $this->_documentID ."' ".
 				"AND `tblDocumentReviewers`.`version` = '". $this->_version ."' ";
@@ -1144,10 +1560,12 @@ class DocumentContent
 			}
 			$queryStr="SELECT `tblDocumentApprovers`.*, `tblDocumentApproveLog`.`status`, ".
 				"`tblDocumentApproveLog`.`comment`, `tblDocumentApproveLog`.`date`, ".
-				"`tblDocumentApproveLog`.`userID` ".
+				"`tblDocumentApproveLog`.`userID`, `tblUsers`.`fullName`, `tblGroups`.`name` AS `groupName` ".
 				"FROM `tblDocumentApprovers` ".
 				"LEFT JOIN `tblDocumentApproveLog` USING (`approveID`) ".
 				"LEFT JOIN `ttapproveid` on `ttapproveid`.`maxLogID` = `tblDocumentApproveLog`.`approveLogID` ".
+				"LEFT JOIN `tblUsers` on `tblUsers`.`id` = `tblDocumentApprovers`.`required`".
+				"LEFT JOIN `tblGroups` on `tblGroups`.`id` = `tblDocumentApprovers`.`required`".
 				"WHERE `ttapproveid`.`maxLogID`=`tblDocumentApproveLog`.`approveLogID` ".
 				"AND `tblDocumentApprovers`.`documentID` = '". $this->_documentID ."' ".
 				"AND `tblDocumentApprovers`.`version` = '". $this->_version ."'";
@@ -1163,11 +1581,13 @@ class DocumentContent
 		GLOBAL $db, $settings;
 
 		$userID = $user->getID();
+		if (!isset($this->_document)) {
+			$this->_document = getDocument($this->_documentID);
+		}
 
 		// Get the list of users and groups with write access to this document.
 		if (!isset($this->_approversList)) {
 			// TODO: error checking.
-			$this->_document = getDocument($this->_documentID);
 			$this->_approversList = $this->_document->getApproversList();
 		}
 		$approved = false;
@@ -1211,6 +1631,9 @@ class DocumentContent
 			return -1;
 		}
 
+		// Add reviewer to event notification table.
+		$this->_document->addNotify($userID, true);
+
 		// Send an email notification to the new reviewer.
 		if ($sendEmail) {
 			$subject = "[Document Review Request]: ".$this->getOriginalFileName().", v. ".$this->_version;
@@ -1220,7 +1643,7 @@ class DocumentContent
 			$message .= "\r\n\r\n\t   Name: ".$this->getOriginalFileName()."\r\n".
 				"\tVersion: ".$this->_version."\r\n".
 				"\tComment: ".$this->getComment()."\r\n".
-				"\t    URL: http".(isset($_SERVER['HTTPS']) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ReviewDocument.php?documentid=".$this->_documentID."&version=".$this->_version."\r\n";
+				"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ReviewDocument.php?documentid=".$this->_documentID."&version=".$this->_version."\r\n";
 			return (Email::toIndividual($requestUser, $user, $subject, $message) < 0 ? -4 : 0);
 		}
 		return 0;
@@ -1230,11 +1653,13 @@ class DocumentContent
 		GLOBAL $db, $settings;
 
 		$groupID = $group->getID();
+		if (!isset($this->_document)) {
+			$this->_document = getDocument($this->_documentID);
+		}
 
 		// Get the list of users and groups with write access to this document.
 		if (!isset($this->_approversList)) {
 			// TODO: error checking.
-			$this->_document = getDocument($this->_documentID);
 			$this->_approversList = $this->_document->getApproversList();
 		}
 		$approved = false;
@@ -1278,6 +1703,9 @@ class DocumentContent
 			return -1;
 		}
 
+		// Add reviewer to event notification table.
+		$this->_document->addNotify($groupID, false);
+
 		// Send an email notification to the new reviewer.
 		if ($sendEmail) {
 			$subject = "[Document Review Request]: ".$this->getOriginalFileName().", v. ".$this->_version;
@@ -1287,7 +1715,7 @@ class DocumentContent
 			$message .="\r\n\r\n\t   Name: ".$this->getOriginalFileName()."\r\n".
 				"\tVersion: ".$this->_version."\r\n".
 				"\tComment: ".$this->getComment()."\r\n".
-				"\t    URL: http".(isset($_SERVER['HTTPS']) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ReviewDocument.php?documentid=".$this->_documentID."&version=".$this->_version."\r\n";
+				"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ReviewDocument.php?documentid=".$this->_documentID."&version=".$this->_version."\r\n";
 			return (Email::toGroup($requestUser, $group, $subject, $message) < 0 ? -4 : 0);
 		}
 		return 0;
@@ -1297,11 +1725,13 @@ class DocumentContent
 		GLOBAL $db, $settings;
 
 		$userID = $user->getID();
+		if (!isset($this->_document)) {
+			$this->_document = getDocument($this->_documentID);
+		}
 
 		// Get the list of users and groups with write access to this document.
 		if (!isset($this->_approversList)) {
 			// TODO: error checking.
-			$this->_document = getDocument($this->_documentID);
 			$this->_approversList = $this->_document->getApproversList();
 		}
 		$approved = false;
@@ -1345,6 +1775,9 @@ class DocumentContent
 			return -1;
 		}
 
+		// Add approver to event notification table.
+		$this->_document->addNotify($userID, true);
+
 		// Send an email notification to the new approver.
 		if ($sendEmail) {
 			$subject = "[Document Approval Request]: ".$this->getOriginalFileName().", v. ".$this->_version;
@@ -1354,7 +1787,7 @@ class DocumentContent
 			$message .= "\r\n\r\n\t   Name: ".$this->getOriginalFileName()."\r\n".
 				"\tVersion: ".$this->_version."\r\n".
 				"\tComment: ".$this->getComment()."\r\n".
-				"\t    URL: http".(isset($_SERVER['HTTPS']) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ApproveDocument.php?documentid=".$this->_documentID."&version=".$this->_version."\r\n";
+				"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ApproveDocument.php?documentid=".$this->_documentID."&version=".$this->_version."\r\n";
 			return (Email::toIndividual($requestUser, $user, $subject, $message) < 0 ? -4 : 0);
 		}
 		return 0;
@@ -1364,11 +1797,13 @@ class DocumentContent
 		GLOBAL $db, $settings;
 
 		$groupID = $group->getID();
+		if (!isset($this->_document)) {
+			$this->_document = getDocument($this->_documentID);
+		}
 
 		// Get the list of users and groups with write access to this document.
 		if (!isset($this->_approversList)) {
 			// TODO: error checking.
-			$this->_document = getDocument($this->_documentID);
 			$this->_approversList = $this->_document->getApproversList();
 		}
 		$approved = false;
@@ -1412,6 +1847,9 @@ class DocumentContent
 			return -1;
 		}
 
+		// Add approver to event notification table.
+		$this->_document->addNotify($groupID, false);
+
 		// Send an email notification to the new approver.
 		if ($sendEmail) {
 			$subject = "[Document Approval Request]: ".$this->getOriginalFileName().", v. ".$this->_version;
@@ -1421,7 +1859,7 @@ class DocumentContent
 			$message .="\r\n\r\n\t   Name: ".$this->getOriginalFileName()."\r\n".
 				"\tVersion: ".$this->_version."\r\n".
 				"\tComment: ".$this->getComment()."\r\n".
-				"\t    URL: http".(isset($_SERVER['HTTPS']) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ApproveDocument.php?documentid=".$this->_documentID."&version=".$this->_version."\r\n";
+				"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ApproveDocument.php?documentid=".$this->_documentID."&version=".$this->_version."\r\n";
 			return (Email::toGroup($requestUser, $group, $subject, $message) < 0 ? -4 : 0);
 		}
 		return 0;
@@ -1464,7 +1902,7 @@ class DocumentContent
 			$message .= "\r\n\r\n\t   Name: ".$this->getOriginalFileName()."\r\n".
 				"\tVersion: ".$this->_version."\r\n".
 				"\tComment: ".$this->getComment()."\r\n".
-				"\t    URL: http".(isset($_SERVER['HTTPS']) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_documentID."\r\n";
+				"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_documentID."\r\n";
 			return (Email::toIndividual($requestUser, $user, $subject, $message) < 0 ? -4 : 0);
 		}
 		return 0;
@@ -1507,7 +1945,7 @@ class DocumentContent
 			$message .= "\r\n\r\n\t   Name: ".$this->getOriginalFileName()."\r\n".
 				"\tVersion: ".$this->_version."\r\n".
 				"\tComment: ".$this->getComment()."\r\n".
-				"\t    URL: http".(isset($_SERVER['HTTPS']) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_documentID."\r\n";
+				"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_documentID."\r\n";
 			return (Email::toGroup($requestUser, $group, $subject, $message) < 0 ? -4 : 0);
 		}
 		return 0;
@@ -1550,7 +1988,7 @@ class DocumentContent
 			$message .= "\r\n\r\n\t   Name: ".$this->getOriginalFileName()."\r\n".
 				"\tVersion: ".$this->_version."\r\n".
 				"\tComment: ".$this->getComment()."\r\n".
-				"\t    URL: http".(isset($_SERVER['HTTPS']) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_documentID."\r\n";
+				"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_documentID."\r\n";
 			return (Email::toIndividual($requestUser, $user, $subject, $message) < 0 ? -4 : 0);
 		}
 		return 0;
@@ -1593,7 +2031,7 @@ class DocumentContent
 			$message .= "\r\n\r\n\t   Name: ".$this->getOriginalFileName()."\r\n".
 				"\tVersion: ".$this->_version."\r\n".
 				"\tComment: ".$this->getComment()."\r\n".
-				"\t    URL: http".(isset($_SERVER['HTTPS']) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_documentID."\r\n";
+				"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$this->_documentID."\r\n";
 			return (Email::toGroup($requestUser, $group, $subject, $message) < 0 ? -4 : 0);
 		}
 		return 0;
