@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -52,6 +52,18 @@ function getFolderPathHTML($folder, $tagAll=false) {
 	return $txtpath;
 }
 
+function getFolderPathPlain($folder) {
+	$path="";
+	$folderPath = $folder->getPath();
+	for ($i = 0; $i  < count($folderPath); $i++) {
+		$path .= $folderPath[$i]->getName();
+		if ($i +1 < count($folderPath))
+			$path .= " / ";
+	}
+	return $path;
+}
+
+
 /**********************************************************************\
 |                            Folder-Klasse                             |
 \**********************************************************************/
@@ -83,14 +95,27 @@ class Folder
 
 	function getName() { return $this->_name; }
 
-	function setName($newName)
-	{
-		GLOBAL $db;
+	function setName($newName) {
+		GLOBAL $db, $user, $settings;
 		
 		$queryStr = "UPDATE tblFolders SET name = '" . $newName . "' WHERE id = ". $this->_id;
-		$res = $db->getResult($queryStr);
-		if (!$res)
+		if (!$db->getResult($queryStr))
 			return false;
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Folder Notification]: ".$this->_name;
+		$message = "Folder '".$this->_name."' has been renamed to '".$newName."'.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\tOld Name: ".$this->_name."\r\n".
+			"\tNew Name: ".$newName."\r\n".
+			"\t  Folder: ".getFolderPathPlain($this)."\r\n".
+			"\t Comment: ".$this->getComment()."\r\n".
+			"\t     URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewFolder.php?folderid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
 		
 		$this->_name = $newName;
 		
@@ -99,17 +124,28 @@ class Folder
 
 	function getComment() { return $this->_comment; }
 
-	function setComment($newComment)
-	{
-		GLOBAL $db;
+	function setComment($newComment) {
+		GLOBAL $db, $user, $settings;
 		
 		$queryStr = "UPDATE tblFolders SET comment = '" . $newComment . "' WHERE id = ". $this->_id;
-		$res = $db->getResult($queryStr);
-		if (!$res)
+		if (!$db->getResult($queryStr))
 			return false;
-		
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Folder Notification]: ".$this->_name;
+		$message = "The comment for folder '".$this->_name."' has been changed.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t       Name: ".$this->_name."\r\n".
+			"\t     Folder: ".getFolderPathPlain($this)."\r\n".
+			"\tNew Comment: ".$newComment."\r\n".
+			"\t     URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewFolder.php?folderid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
 		$this->_comment = $newComment;
-		
 		return true;
 	}
 
@@ -127,10 +163,9 @@ class Folder
 		return $this->_parent;
 	}
 
-	function setParent($newParent)
-	{
-		global $db, $settings;
-		
+	function setParent($newParent) {
+		global $db, $user, $settings;
+
 		if ($this->_id == $settings->_rootFolderID || !isset($this->_parentID) || ($this->_parentID == null) || ($this->_parentID == "") || ($this->_parentID == 0)) {
 			return false;
 		}
@@ -164,6 +199,20 @@ class Folder
 			$res = $db->getResult($queryStr);
 		}
 
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Folder Notification]: ".$this->_name;
+		$message = "Folder '".$this->_name."' has been moved.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->_name."\r\n".
+			"\t Folder: ".getFolderPathPlain($this)."\r\n".
+			"\tComment: ".$this->_comment."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewFolder.php?folderid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
 		return true;
 	}
 
@@ -174,16 +223,31 @@ class Folder
 		return $this->_owner;
 	}
 
-	function setOwner($user)
-	{
-		GLOBAL $db;
-		
-		$queryStr = "UPDATE tblFolders set owner = " . $user->getID() . " WHERE id = " . $this->_id;
+	function setOwner($newOwner) {
+		GLOBAL $db, $user, $settings;
+
+		$oldOwner = $this->getOwner();
+
+		$queryStr = "UPDATE tblFolders set owner = " . $newOwner->getID() . " WHERE id = " . $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
-		
-		$this->_ownerID = $user->getID();
-		$this->_owner = $user;
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Folder Notification]: ".$this->_name;
+		$message = "Ownership has changed for folder '".$this->_name."'. Old owner: '".$oldOwner->getFullName()."'. New Owner: '".$newOwner->getFullName()."'";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->_name."\r\n".
+			"\t Folder: ".getFolderPathPlain($this)."\r\n".
+			"\tComment: ".$this->_comment."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewFolder.php?folderid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
+		$this->_ownerID = $newOwner->getID();
+		$this->_owner = $newOwner;
 		return true;
 	}
 
@@ -199,31 +263,85 @@ class Folder
 		return $this->_defaultAccess;
 	}
 
-	function setDefaultAccess($mode)
-	{
-		GLOBAL $db;
-		
+	function setDefaultAccess($mode) {
+		GLOBAL $db, $user, $settings;
+
 		$queryStr = "UPDATE tblFolders set defaultAccess = " . $mode . " WHERE id = " . $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
-		
-		$this->_defaulAccess = $mode;
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Folder Notification]: ".$this->_name;
+		$message = "Default access privileges have changed for folder '".$this->_name."'.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->_name."\r\n".
+			"\t Folder: ".getFolderPathPlain($this)."\r\n".
+			"\tComment: ".$this->_comment."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewFolder.php?folderid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
+		$this->_defaultAccess = $mode;
+
+		// If any of the notification subscribers no longer have read access,
+		// remove their subscription.
+		foreach ($this->_notifyList["users"] as $u) {
+			if ($this->getAccessMode($u) < M_READ) {
+				$this->removeNotify($u->getID(), true);
+			}
+		}
+		foreach ($this->_notifyList["groups"] as $g) {
+			if ($this->getGroupAccessMode($g) < M_READ) {
+				$this->removeNotify($g->getID(), false);
+			}
+		}
+
 		return true;
 	}
 
 	function inheritsAccess() { return $this->_inheritAccess; }
 
-	function setInheritAccess($inheritAccess)
-	{
-		GLOBAL $db;
-		
+	function setInheritAccess($inheritAccess) {
+		GLOBAL $db, $user, $settings;
+
 		$inheritAccess = ($inheritAccess) ? "1" : "0";
-		
+
 		$queryStr = "UPDATE tblFolders SET inheritAccess = " . $inheritAccess . " WHERE id = " . $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
 		$this->_inheritAccess = $inheritAccess;
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Folder Notification]: ".$this->_name;
+		$message = "Default access privileges have changed for folder '".$this->_name."'. Access is ".($inheritAccess ? "now" : "no longer")." inherited.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->_name."\r\n".
+			"\t Folder: ".getFolderPathPlain($this)."\r\n".
+			"\tComment: ".$this->_comment."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewFolder.php?folderid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
+		// If any of the notification subscribers no longer have read access,
+		// remove their subscription.
+		foreach ($this->_notifyList["users"] as $u) {
+			if ($this->getAccessMode($u) < M_READ) {
+				$this->removeNotify($u->getID(), true);
+			}
+		}
+		foreach ($this->_notifyList["groups"] as $g) {
+			if ($this->getGroupAccessMode($g) < M_READ) {
+				$this->removeNotify($g->getID(), false);
+			}
+		}
+
 		return true;
 	}
 
@@ -241,8 +359,7 @@ class Folder
 		return true;
 	}
 
-	function getSubFolders()
-	{
+	function getSubFolders() {
 		GLOBAL $db;
 		
 		if (!isset($this->_subFolders))
@@ -260,32 +377,44 @@ class Folder
 		return $this->_subFolders;
 	}
 
-	function addSubFolder($name, $comment, $owner, $sequence)
-	{
-		GLOBAL $db;
-		
+	function addSubFolder($name, $comment, $owner, $sequence) {
+		GLOBAL $db, $user, $settings;
+
 		//inheritAccess = true, defaultAccess = M_READ
 		$queryStr = "INSERT INTO tblFolders (name, parent, comment, owner, inheritAccess, defaultAccess, sequence) ".
 					"VALUES ('".$name."', ".$this->_id.", '".$comment."', ".$owner->getID().", 1, ".M_READ.", ".$sequence.")";
-		$res = $db->getResult($queryStr);
-		if (!$res)
+		if (!$db->getResult($queryStr))
 			return false;
-		
+		$newFolder = getFolder($db->getInsertID());
 		unset($this->_subFolders);
-		
-		return getFolder($db->getInsertID());
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Folder Notification]: ".$this->_name;
+		$message = "A new sub-folder, '".$name."', has been added to '".$this->_name."'.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\tSub-folder Name: ".$name."\r\n".
+			"\t   Path: ".getFolderPathPlain($newFolder)."\r\n".
+			"\tComment: ".$comment."\r\n".
+			"\t  Owner: ".$owner->getFullName()."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewFolder.php?folderid=".$newFolder->getID()."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
+		return $newFolder;
 	}
 
 	/**
 	 * Gibt ein Array mit allen Eltern, "Groelter" usw bis zum RootFolder zurck
 	 * Der Ordner selbst ist das letzte Element dieses Arrays
 	 */
-	function getPath()
-	{
-		if (!isset($this->_parentID) || ($this->_parentID == "") || ($this->_parentID == 0))
+	function getPath() {
+		if (!isset($this->_parentID) || ($this->_parentID == "") || ($this->_parentID == 0)) {
 			return array($this);
-		else
-		{
+		}
+		else {
 			$res = $this->getParent();
 			if (!$res) return false;
 			
@@ -338,9 +467,8 @@ class Folder
 		return $this->_documents;
 	}
 
-	function addDocument($name, $comment, $expires, $owner, $keywords, $tmpFile, $orgFileName, $fileType, $mimeType, $sequence, $reviewers=array(), $approvers=array())
-	{
-		GLOBAL $db;
+	function addDocument($name, $comment, $expires, $owner, $keywords, $tmpFile, $orgFileName, $fileType, $mimeType, $sequence, $reviewers=array(), $approvers=array()) {
+		GLOBAL $db, $user, $settings;
 		
 		$expires = (!$expires) ? 0 : $expires;
 
@@ -368,12 +496,31 @@ class Folder
 			return false;
 		}
 
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Folder Notification]: ".$this->_name;
+		$message = "A new document has been added to folder '".$this->_name."'";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\nDocument Information\r\n\r\n".
+			"\t   Name: ".$name."\r\n".
+			"\t Folder: ".getFolderPathPlain($this)."\r\n".
+			"\tComment: ".$comment."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewDocument.php?documentid=".$document->getID()."\r\n";
+		$message .= "\r\nFolder Information\r\n\r\n".
+			"\t   Name: ".$this->_name."\r\n".
+			"\t Folder: ".getFolderPathPlain($this)."\r\n".
+			"\tComment: ".$this->_comment."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewFolder.php?folderid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
 		return array($document, $res);
 	}
 
-	function remove()
-	{
-		global $db, $settings;
+	function remove() {
+		global $db, $user, $settings;
 
 		// Do not delete the root folder.
 		if ($this->_id == $settings->_rootFolderID || !isset($this->_parentID) || ($this->_parentID == null) || ($this->_parentID == "") || ($this->_parentID == 0)) {
@@ -405,6 +552,21 @@ class Folder
 		$queryStr = "DELETE FROM tblACLs WHERE target = ". $this->_id. " AND targetType = " . T_FOLDER;
 		if (!$db->getResult($queryStr))
 			return false;
+
+		// Send notification to subscribers.
+		$this->getNotifyList();
+		$subject = "[Folder Notification]: ".$this->_name;
+		$message = "Folder, '".$this->_name."', has been deleted.";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n".
+			"\t   Path: ".getFolderPathPlain($this)."\r\n".
+			"\tComment: ".$this->_comment."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewFolder.php?folderid=".$this->_id."\r\n";
+		Email::toList($user, $this->_notifyList["users"], $subject, $message);
+		foreach ($this->_notifyList["groups"] as $grp) {
+			Email::toGroup($user, $grp, $subject, $message);
+		}
+
 		$queryStr = "DELETE FROM tblNotify WHERE target = ". $this->_id. " AND targetType = " . T_FOLDER;
 		if (!$db->getResult($queryStr))
 			return false;
@@ -464,46 +626,62 @@ class Folder
 		return true;
 	}
 
-	function addAccess($mode, $userOrGroupID, $isUser)
-	{
+	function addAccess($mode, $userOrGroupID, $isUser) {
 		GLOBAL $db;
-		
+
 		$userOrGroup = ($isUser) ? "userID" : "groupID";
-		
+
 		$queryStr = "INSERT INTO tblACLs (target, targetType, ".$userOrGroup.", mode) VALUES 
 					(".$this->_id.", ".T_FOLDER.", " . $userOrGroupID . ", " .$mode. ")";
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
 		unset($this->_accessList);
+
+		// Update the notify list, if necessary.
+		if ($mode == M_NONE) {
+			$this->removeNotify($userOrGroupID, $isUser);
+		}
+
 		return true;
 	}
 
-	function changeAccess($newMode, $userOrGroupID, $isUser)
-	{
+	function changeAccess($newMode, $userOrGroupID, $isUser) {
 		GLOBAL $db;
-		
+
 		$userOrGroup = ($isUser) ? "userID" : "groupID";
-		
+
 		$queryStr = "UPDATE tblACLs SET mode = " . $newMode . " WHERE targetType = ".T_FOLDER." AND target = " . $this->_id . " AND " . $userOrGroup . " = " . $userOrGroupID;
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
 		unset($this->_accessList);
+
+		// Update the notify list, if necessary.
+		if ($newMode == M_NONE) {
+			$this->removeNotify($userOrGroupID, $isUser);
+		}
+
 		return true;
 	}
 
-	function removeAccess($userOrGroupID, $isUser)
-	{
+	function removeAccess($userOrGroupID, $isUser) {
 		GLOBAL $db;
-		
+
 		$userOrGroup = ($isUser) ? "userID" : "groupID";
-		
+
 		$queryStr = "DELETE FROM tblACLs WHERE targetType = ".T_FOLDER." AND target = ".$this->_id." AND ".$userOrGroup." = " . $userOrGroupID;
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
 		unset($this->_accessList);
+
+		// Update the notify list, if necessary.
+		$mode = ($isUser ? $this->getAccessMode(getUser($userOrGroupID)) : $this->getGroupAccessMode(getGroup($userOrGroupID)));
+		if ($mode == M_NONE) {
+			$this->removeNotify($userOrGroupID, $isUser);
+		}
+
 		return true;
 	}
 
@@ -551,15 +729,15 @@ class Folder
 			}
 		}
 		*/
-		
+
 		$highestPrivileged = M_NONE;
-		
+
 		//ACLs durchforsten
 		$foundInACL = false;
 		$accessList = $this->getAccessList();
 		if (!$accessList)
 			return false;
-			
+
 		foreach ($accessList["users"] as $userAccess)
 		{
 			if ($userAccess->getUserID() == $user->getID())
@@ -584,25 +762,22 @@ class Folder
 		}
 		if ($foundInACL)
 			return $highestPrivileged;
-		
+
 		//Standard-Berechtigung verwenden
 		return $this->getDefaultAccess();
 	}
 
-	function getGroupAccessMode($group)
-	{
+	function getGroupAccessMode($group) {
 		GLOBAL $settings;
-		
+
 		$highestPrivileged = M_NONE;
 		$foundInACL = false;
 		$accessList = $this->getAccessList();
 		if (!$accessList)
 			return false;
-			
-		foreach ($accessList["groups"] as $groupAccess)
-		{
-			if ($groupAccess->getGroupID() == $group->getID())
-			{
+
+		foreach ($accessList["groups"] as $groupAccess) {
+			if ($groupAccess->getGroupID() == $group->getID()) {
 				$foundInACL = true;
 				if ($groupAccess->getMode() > $highestPrivileged)
 					$highestPrivileged = $groupAccess->getMode();
@@ -612,7 +787,7 @@ class Folder
 		}
 		if ($foundInACL)
 			return $highestPrivileged;
-		
+
 		//Standard-Berechtigung verwenden
 		return $this->getDefaultAccess();
 	}
@@ -622,12 +797,12 @@ class Folder
 		if (!isset($this->_notifyList))
 		{
 			GLOBAL $db;
-			
+
 			$queryStr ="SELECT * FROM tblNotify WHERE targetType = " . T_FOLDER . " AND target = " . $this->_id;
 			$resArr = $db->getResultArray($queryStr);
 			if (is_bool($resArr) && $resArr == false)
 				return false;
-			
+
 			$this->_notifyList = array("groups" => array(), "users" => array());
 			foreach ($resArr as $row)
 			{
@@ -640,32 +815,220 @@ class Folder
 		return $this->_notifyList;
 	}
 
-	function addNotify($userOrGroupID, $isUser)
-	{
-		GLOBAL $db;
-		
+	function addNotify($userOrGroupID, $isUser) {
+
+		// Return values:
+		// -1: Invalid User/Group ID.
+		// -2: Target User / Group does not have read access.
+		// -3: User is already subscribed.
+		// -4: Database / internal error.
+		//  0: Update successful.
+
+		GLOBAL $db, $settings, $user;
+
 		$userOrGroup = ($isUser) ? "userID" : "groupID";
-		
+
+		//
+		// Verify that user / group exists.
+		//
+		$obj = ($isUser ? getUser($userOrGroupID) : getGroup($userOrGroupID));
+		if (!is_object($obj)) {
+			return -1;
+		}
+
+		//
+		// Verify that the requesting user has permission to add the target to
+		// the notification system.
+		//
+		if ($user->getID() == $settings->_guestID) {
+			return -2;
+		}
+		if (!$user->isAdmin()) {
+			if ($isUser) {
+				if ($user->getID() != $obj->getID()) {
+					return -2;
+				}
+			}
+			else {
+				if (!$obj->isMember($user)) {
+					return -2;
+				}
+			}
+		}
+
+		//
+		// Verify that user / group has read access to the document.
+		//
+		if ($isUser) {
+			// Users are straightforward to check.
+			if ($this->getAccessMode($obj) < M_READ) {
+				return -2;
+			}
+		}
+		else {
+			// Groups are a little more complex.
+			if ($this->getDefaultAccess() >= M_READ) {
+				// If the default access is at least READ-ONLY, then just make sure
+				// that the current group has not been explicitly excluded.
+				$acl = $this->getAccessList(M_NONE, O_EQ);
+				$found = false;
+				foreach ($acl["groups"] as $group) {
+					if ($group->getGroupID() == $userOrGroupID) {
+						$found = true;
+						break;
+					}
+				}
+				if ($found) {
+					return -2;
+				}
+			}
+			else {
+				// The default access is restricted. Make sure that the group has
+				// been explicitly allocated access to the document.
+				$acl = $this->getAccessList(M_READ, O_GTEQ);
+				if (is_bool($acl)) {
+					return -4;
+				}
+				$found = false;
+				foreach ($acl["groups"] as $group) {
+					if ($group->getGroupID() == $userOrGroupID) {
+						$found = true;
+						break;
+					}
+				}
+				if (!$found) {
+					return -2;
+				}
+			}
+		}
+		//
+		// Check to see if user/group is already on the list.
+		//
+		$queryStr = "SELECT * FROM `tblNotify` WHERE `tblNotify`.`target` = '".$this->_id."' ".
+			"AND `tblNotify`.`targetType` = '".T_FOLDER."' ".
+			"AND `tblNotify`.`".$userOrGroup."` = '".$userOrGroupID."'";
+		$resArr = $db->getResultArray($queryStr);
+		if (is_bool($resArr)) {
+			return -4;
+		}
+		if (count($resArr)>0) {
+			return -3;
+		}
+
 		$queryStr = "INSERT INTO tblNotify (target, targetType, " . $userOrGroup . ") VALUES (" . $this->_id . ", " . T_FOLDER . ", " . $userOrGroupID . ")";
 		if (!$db->getResult($queryStr))
-			return false;
-		
+			return -4;
+
+		// Email user / group, informing them of subscription.
+		$path="";
+		$folderPath = $this->getPath();
+		for ($i = 0; $i  < count($folderPath); $i++) {
+			$path .= $folderPath[$i]->getName();
+			if ($i +1 < count($folderPath))
+				$path .= " / ";
+		}
+		$subject = "[Folder Notification]: ".$this->getName();
+		$message = ($isUser ? "You have" : "The group '".$obj->getName()."' has")." been added to the event notification system for the following folder: ";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->getName()."\r\n".
+			"\t   Path: ".$path."\r\n".
+			"\tComment: ".$this->getComment()."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewFolder.php?folderid=".$this->_id."\r\n";
+
+		if ($isUser) {
+			Email::toIndividual($user, $obj, $subject, $message);
+		}
+		else {
+			Email::toGroup($user, $obj, $subject, $message);
+		}
+
 		unset($this->_notifyList);
-		return true;
+		return 0;
 	}
 
-	function removeNotify($userOrGroupID, $isUser)
-	{
-		GLOBAL $db;
+	function removeNotify($userOrGroupID, $isUser) {
+
+		// Return values:
+		// -1: Invalid User/Group ID.
+		// -3: User is not subscribed. No action taken.
+		// -4: Database / internal error.
+		//  0: Update successful.
+
+		GLOBAL $db, $settings, $user;
 		
+		//
+		// Verify that user / group exists.
+		//
+		$obj = ($isUser ? getUser($userOrGroupID) : getGroup($userOrGroupID));
+		if (!is_object($obj)) {
+			return -1;
+		}
+
 		$userOrGroup = ($isUser) ? "userID" : "groupID";
-		
+
+		//
+		// Verify that the requesting user has permission to add the target to
+		// the notification system.
+		//
+		if ($user->getID() == $settings->_guestID) {
+			return -2;
+		}
+		if (!$user->isAdmin()) {
+			if ($isUser) {
+				if ($user->getID() != $obj->getID()) {
+					return -2;
+				}
+			}
+			else {
+				if (!$obj->isMember($user)) {
+					return -2;
+				}
+			}
+		}
+
+		//
+		// Check to see if the target is in the database.
+		//
+		$queryStr = "SELECT * FROM `tblNotify` WHERE `tblNotify`.`target` = '".$this->_id."' ".
+			"AND `tblNotify`.`targetType` = '".T_FOLDER."' ".
+			"AND `tblNotify`.`".$userOrGroup."` = '".$userOrGroupID."'";
+		$resArr = $db->getResultArray($queryStr);
+		if (is_bool($resArr)) {
+			return -4;
+		}
+		if (count($resArr)==0) {
+			return -3;
+		}
+
 		$queryStr = "DELETE FROM tblNotify WHERE target = " . $this->_id . " AND targetType = " . T_FOLDER . " AND " . $userOrGroup . " = " . $userOrGroupID;
 		if (!$db->getResult($queryStr))
-			return false;
-		
+			return -4;
+
+		// Email user / group, informing them of subscription.
+		$path="";
+		$folderPath = $this->getPath();
+		for ($i = 0; $i  < count($folderPath); $i++) {
+			$path .= $folderPath[$i]->getName();
+			if ($i +1 < count($folderPath))
+				$path .= " / ";
+		}
+		$subject = "[Folder Notification]: ".$this->getName();
+		$message = ($isUser ? "You have" : "The group '".$obj->getName()."' has")." been removed from the event notification system for the following folder: ";
+		$message = wordwrap ($message, 72, "\r\n");
+		$message .= "\r\n\r\n\t   Name: ".$this->getName()."\r\n".
+			"\t   Path: ".$path."\r\n".
+			"\tComment: ".$this->getComment()."\r\n".
+			"\t    URL: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'].$settings->_httpRoot."out/out.ViewFolder.php?folderid=".$this->_id."\r\n";
+
+		if ($isUser) {
+			Email::toIndividual($user, $obj, $subject, $message);
+		}
+		else {
+			Email::toGroup($user, $obj, $subject, $message);
+		}
+
 		unset($this->_notifyList);
-		return true;
+		return 0;
 	}
 
 	function getApproversList() {
