@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@ if (!isset($theme) || strlen($theme)==0) {
 	$theme = $settings->_theme;
 }
 if (strlen($theme)==0) {
-	$theme="default";
+	$theme="blue";
 }
 
 $icons = array();
@@ -79,16 +79,29 @@ class UI {
 		echo "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\"\n".
 			"\"http://www.w3.org/TR/html4/strict.dtd\">\n";
 		echo "<html>\n<head>\n";
-		echo "<link rel=\"STYLESHEET\" type=\"text/css\" href=\"../styles/".$theme."/style.css\">\n";
+		echo "<link rel=\"STYLESHEET\" type=\"text/css\" href=\"../styles/".$theme."/style.css\"/>\n";
+		echo "<link rel='shortcut icon' href='../styles/".$theme."/favicon.ico' type='image/x-icon'/>";
+
 		echo "<title>".(strlen($settings->_siteName)>0 ? $settings->_siteName : "MyDMS").(strlen($title)>0 ? ": " : "").$title."</title>\n";
 		echo "</head>\n";
 		echo "<body".(strlen($bodyClass)>0 ? " class=\"".$bodyClass."\"" : "").">\n";
 	}
 
 	function htmlEndPage() {
+		UI::footNote();
 		echo "</body>\n</html>\n";
 	}
 
+	function footNote() {
+		global $settings;
+
+		if (isset($settings->_footNote) && strlen((string)$settings->_footNote)>0) {
+			echo "<div class=\"footNote\">".(string)$settings->_footNote."</div>";
+		}
+		return;
+	}
+
+
 	function globalBanner() {
 		global $settings;
 
@@ -97,7 +110,7 @@ class UI {
 		echo "<div class=\"siteName\">".
 			(strlen($settings->_siteName)>0 ? $settings->_siteName : "MyDMS").
 			"</div>\n";
-		echo "<div style=\"clear: both; height: 0px;\">&nbsp;</div>\n".
+		echo "<div style=\"clear: both; height: 0px; font-size:0;\">&nbsp;</div>\n".
 			"</div>\n";
 		return;
 	}
@@ -129,7 +142,7 @@ class UI {
 		echo "<span class=\"absSpacerNorm\"></span>\n";
 		echo "<div id=\"signatory\">Signed in as ".$user->getFullName().
 			" (<a href=\"../op/op.Logout.php\">Sign out</a>).</div>\n";
-		echo "<div style=\"clear: both; height: 0px;\">&nbsp;</div>\n".
+		echo "<div style=\"clear: both; height: 0px; font-size:0;\">&nbsp;</div>\n".
 			"</div>\n";
 		return;
 	}
@@ -138,7 +151,14 @@ class UI {
 		global $settings, $user;
 
 		echo "<div class=\"headingContainer\">\n";
+		// This spacer span is an awful hack, but it is the only way I know to
+		// get the spacer to match the mainheading content's size.
+		echo "<span class=\"absSpacerTitle\">".($settings->_titleDisplayHack ? $pageTitle : "")."</span>\n";
+		echo "<div class=\"mainHeading\">".$pageTitle."</div>\n";
+		echo "<div style=\"clear: both; height: 0px; font-size:0;\"></div>\n</div>\n";
+
 		if ($pageType!=null && strcasecmp($pageType, "noNav")) {
+			echo "<div class=\"localNavContainer\">\n";
 			switch ($pageType) {
 				case "view_folder":
 					UI::folderNavigationBar($extra);
@@ -165,17 +185,14 @@ class UI {
 					UI::adminToolsNavigationBar();
 					break;
 			}
+			echo "<div style=\"clear: both; height: 0px; font-size:0;\"></div>\n</div>\n";
 		}
-		// This spacer span is an awful hack, but it is the only way I know to
-		// get the spacer to match the mainheading content's size.
-		echo "<span class=\"absSpacerTitle\">".($settings->_titleDisplayHack ? $pageTitle : "")."</span>\n";
-		echo "<div class=\"mainHeading\">".$pageTitle."</div>\n";
-		echo "<div style=\"clear: both; height: 0px;\"></div>\n</div>\n";
+
 		return;
 	}
 
 	function folderNavigationBar($folder) {
-		global $user, $theme;
+		global $user, $settings, $theme;
 
 		if (!is_object($folder) || strcasecmp(get_class($folder), "Folder")) {
 			echo "<ul class=\"localNav\">\n";
@@ -185,7 +202,10 @@ class UI {
 		$accessMode = $folder->getAccessMode($user);
 		$folderID = $folder->getID();
 		echo "<ul class=\"localNav\">\n";
-		if ($accessMode >= M_READWRITE) {
+		if ($accessMode == M_READ && $user->getID() != $settings->_guestID) {
+			echo "<li id=\"first\"><a href=\"../out/out.FolderNotify.php?folderid=". $folderID ."\">".getMLText("edit_folder_notify")."</a></li>\n";
+		}
+		else if ($accessMode >= M_READWRITE) {
 			echo "<li id=\"first\"><a href=\"../out/out.AddSubFolder.php?folderid=". $folderID ."\">".getMLText("add_subfolder")."</a></li>\n";
 			echo "<li><a href=\"../out/out.AddDocument.php?folderid=". $folderID ."\">".getMLText("add_document")."</a></li>\n";
 			echo "<li><a href=\"../out/out.EditFolder.php?folderid=". $folderID ."\">".getMLText("edit_folder_props")."</a></li>\n";
@@ -201,13 +221,16 @@ class UI {
 	}
 
 	function documentNavigationBar()	{
-		global $user, $document;
+		global $user, $settings, $document;
 
 		$accessMode = $document->getAccessMode($user);
 		$docid=".php?documentid=" . $document->getID();
 
 		echo "<ul class=\"localNav\">\n";
-		if ($accessMode >= M_READWRITE) {
+		if ($accessMode == M_READ && $user->getID() != $settings->_guestID) {
+			echo "<li id=\"first\"><a href=\"../out/out.DocumentNotify". $docid ."\">".getMLText("edit_document_notify")."</a></li>";
+		}
+		else if ($accessMode >= M_READWRITE) {
 			if (!$document->isLocked()) {
 				echo "<li id=\"first\"><a href=\"../out/out.UpdateDocument". $docid ."\">".getMLText("update_document")."</a></li>";
 				echo "<li><a href=\"../op/op.LockDocument". $docid ."\">".getMLText("lock_document")."</a></li>";
@@ -242,6 +265,7 @@ class UI {
 		echo "<li id=\"first\"><a href=\"../out/out.MyDocuments.php\">".getMLText("my_documents")."</a></li>\n";
 		echo "<li><a href=\"../out/out.ReviewSummary.php\">".getMLText("review_summary")."</a></li>\n";
 		echo "<li><a href=\"../out/out.ApprovalSummary.php\">".getMLText("approval_summary")."</a></li>\n";
+		echo "<li><a href=\"../out/out.ManageNotify.php\">".getMLText("edit_document_notify")."</a></li>\n";
 		echo "<li><a href=\"../out/out.EditUserData.php\">".getMLText("my_account")."</a></li>\n";
 		echo "</ul>\n";
 		return;
@@ -302,6 +326,54 @@ class UI {
 		return;
 	}
 
+	function pageList($pageNumber, $totalPages, $baseURI, $params) {
+
+		if (!is_numeric($pageNumber) || !is_numeric($totalPages) || $totalPages<1) {
+			return;
+		}
+
+		// Construct the basic URI based on the $_GET array. One could use a
+		// regular expression to strip out the pg (page number) variable to
+		// achieve the same effect. This seems to be less haphazard though...
+		$resultsURI = $baseURI;
+		$first=true;
+		foreach ($params as $key=>$value) {
+			// Don't include the page number in the basic URI. This is added in
+			// during the list display loop.
+			if (!strcasecmp($key, "pg")) {
+				continue;
+			}
+			if (is_array($value)) {
+				foreach ($value as $subvalue) {
+					$resultsURI .= ($first ? "?" : "&").$key."%5B%5D=".$subvalue;
+					$first = false;
+				}
+			}
+			else {
+					$resultsURI .= ($first ? "?" : "&").$key."=".$value;
+			}
+			$first = false;
+		}
+
+		echo "<ul class=\"pageList\">";
+		echo "<li id=\"first\">".getMLText("results_page").": ";
+		for ($i = 1; $i  <= $totalPages; $i++) {
+			if ($i == $pageNumber) {
+				echo "<li>".$i;
+			}
+			else {
+				echo "<li><a href=\"".$resultsURI.($first ? "?" : "&")."pg=".$i."\">".$i."</a>";
+			}
+		}
+		if ($totalPages>1) {
+			echo "<li><a href=\"".$resultsURI.($first ? "?" : "&")."pg=all\">".getMLText("all_pages")."</a>";
+		}
+		echo "</ul>";
+		
+
+		return;
+	}
+
 	function contentContainer($content) {
 
 		echo "<div class=\"contentContainer\">\n";
@@ -353,7 +425,7 @@ class UI {
 		<script language="JavaScript">
 		var openDlg;
 		function chooseDoc() {
-			openDlg = open("../out/out.DocumentChooser.php?folderid=<?php echo $settings->_rootFolderID?>&form=<?php echo urlencode($formName)?>", "openDlg", "width=300,height=450,scrollbars=yes,resizable=yes,status=yes");
+			openDlg = open("../out/out.DocumentChooser.php?folderid=<?php echo $settings->_rootFolderID?>&form=<?php echo urlencode($formName)?>", "openDlg", "width=480,height=480,scrollbars=yes,resizable=yes,status=yes");
 		}
 		</script>
 		<?php
@@ -430,7 +502,7 @@ class UI {
 		<script language="JavaScript">
 		var openDlg;
 		function chooseDoc() {
-			openDlg = open("out.FolderChooser.php?form=<?php echo $formName?>&mode=<?php echo $accessMode?>&exclude=<?php echo $exclude?>&folderid=<?php echo $settings->_rootFolderID?>", "openDlg", "width=300,height=450,scrollbars=yes,resizable=yes,status=yes");
+			openDlg = open("out.FolderChooser.php?form=<?php echo $formName?>&mode=<?php echo $accessMode?>&exclude=<?php echo $exclude?>&folderid=<?php echo $settings->_rootFolderID?>", "openDlg", "width=480,height=480,scrollbars=yes,resizable=yes,status=yes");
 		}
 		</script>
 		<?php
