@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -143,7 +143,7 @@ class DatabaseAccess
 
 	function createTemporaryTable($tableName, $override=false) {
 		if (!strcasecmp($tableName, "ttreviewid")) {
-			$queryStr = "CREATE TEMPORARY TABLE IF NOT EXISTS `ttreviewid` (PRIMARY KEY (`reviewID`)) ".
+			$queryStr = "CREATE TEMPORARY TABLE IF NOT EXISTS `ttreviewid` (PRIMARY KEY (`reviewID`), INDEX (`maxLogID`)) ".
 				"SELECT `tblDocumentReviewLog`.`reviewID`, ".
 				"MAX(`tblDocumentReviewLog`.`reviewLogID`) AS `maxLogID` ".
 				"FROM `tblDocumentReviewLog` ".
@@ -165,7 +165,7 @@ class DatabaseAccess
 			return $this->_ttreviewid;
 		}
 		else if (!strcasecmp($tableName, "ttapproveid")) {
-			$queryStr = "CREATE TEMPORARY TABLE IF NOT EXISTS `ttapproveid` (PRIMARY KEY (`approveID`)) ".
+			$queryStr = "CREATE TEMPORARY TABLE IF NOT EXISTS `ttapproveid` (PRIMARY KEY (`approveID`), INDEX (`maxLogID`)) ".
 				"SELECT `tblDocumentApproveLog`.`approveID`, ".
 				"MAX(`tblDocumentApproveLog`.`approveLogID`) AS `maxLogID` ".
 				"FROM `tblDocumentApproveLog` ".
@@ -187,7 +187,7 @@ class DatabaseAccess
 			return $this->_ttapproveid;
 		}
 		else if (!strcasecmp($tableName, "ttstatid")) {
-			$queryStr = "CREATE TEMPORARY TABLE IF NOT EXISTS `ttstatid` (PRIMARY KEY (`statusID`)) ".
+			$queryStr = "CREATE TEMPORARY TABLE IF NOT EXISTS `ttstatid` (PRIMARY KEY (`statusID`), INDEX (`maxLogID`)) ".
 				"SELECT `tblDocumentStatusLog`.`statusID`, ".
 				"MAX(`tblDocumentStatusLog`.`statusLogID`) AS `maxLogID` ".
 				"FROM `tblDocumentStatusLog` ".
@@ -209,7 +209,7 @@ class DatabaseAccess
 			return $this->_ttstatid;
 		}
 		else if (!strcasecmp($tableName, "ttcontentid")) {
-			$queryStr = "CREATE TEMPORARY TABLE `ttcontentid` (PRIMARY KEY (`document`)) ".
+			$queryStr = "CREATE TEMPORARY TABLE `ttcontentid` (PRIMARY KEY (`document`), INDEX (`maxVersion`)) ".
 				"SELECT `tblDocumentContent`.`document`, ".
 				"MAX(`tblDocumentContent`.`version`) AS `maxVersion` ".
 				"FROM `tblDocumentContent` ".
