@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@ include("../inc/inc.DBAccess.php");
 include("../inc/inc.FileUtils.php");
 include("../inc/inc.Language.php");
 include("../inc/inc.ClassUI.php");
+include("../inc/inc.ClassEmail.php");
 include("../inc/inc.Authentication.php");
 
 if (!isset($_GET["action"]) || (strcasecmp($_GET["action"], "delnotify") && strcasecmp($_GET["action"],"addnotify"))) {
@@ -80,7 +81,7 @@ UI::htmlStartPage(getMLText("document_title", array("documentname" => $document-
 UI::globalNavigation($folder);
 UI::pageNavigation($docPathHTML, "view_document");
 
-if ($document->getAccessMode($user) < M_READWRITE) {
+if ($document->getAccessMode($user) < M_READ) {
 	UI::contentHeading(getMLText("edit_existing_notify") . ": " . $document->getName());
 	UI::contentContainer(getMLText("access_denied"));
 	UI::htmlEndPage();
@@ -94,22 +95,27 @@ UI::contentContainerStart();
 if ($action == "delnotify")	{
 	printMLText("deleting_document_notify");
 	if (isset($userid)) {
-		if (!is_object(getUser($userid))) {
-			printMLText("unknown_user");
-			UI::contentContainerEnd();
-			UI::htmlEndPage();
-			exit;
-		}
-		$document->removeNotify($userid, true);
+		$res = $document->removeNotify($userid, true);
 	}
 	else if (isset($groupid)) {
-		if (!is_object(getGroup($groupid))) {
-			printMLText("unknown_group");
-			UI::contentContainerEnd();
-			UI::htmlEndPage();
-			exit;
-		}
-		$document->removeNotify($groupid, false);
+		$res = $document->removeNotify($groupid, false);
+	}
+	switch ($res) {
+		case -1:
+			print (isset($userid) ? getMLText("unknown_user") : getMLText("unknown_group"));
+			break;
+		case -2:
+			printMLText("access_denied");
+			break;
+		case -3:
+			printMLText("already_subscribed");
+			break;
+		case -4:
+			printMLText("internal_error");
+			break;
+		case 0:
+			printMLText("op_finished");
+			break;
 	}
 }
 	
@@ -117,25 +123,47 @@ if ($action == "delnotify")	{
 else if ($action == "addnotify") {
 	printMLText("adding_document_notify");
 	if ($userid != -1) {
-		if (!is_object(getUser($userid))) {
-			printMLText("unknown_user");
-			UI::contentContainerEnd();
-			UI::htmlEndPage();
-			exit;
+		$res = $document->addNotify($userid, true);
+		switch ($res) {
+			case -1:
+				printMLText("unknown_user");
+				break;
+			case -2:
+				printMLText("access_denied");
+				break;
+			case -3:
+				printMLText("already_subscribed");
+				break;
+			case -4:
+				printMLText("internal_error");
+				break;
+			case 0:
+				printMLText("op_finished");
+				break;
 		}
-		$document->addNotify($userid, true);
 	}
 	if ($groupid != -1) {
-		if (!is_object(getGroup($groupid))) {
-			printMLText("unknown_group");
-			UI::contentContainerEnd();
-			UI::htmlEndPage();
-			exit;
+		$res = $document->addNotify($groupid, false);
+		switch ($res) {
+			case -1:
+				printMLText("unknown_group");
+				break;
+			case -2:
+				printMLText("access_denied");
+				break;
+			case -3:
+				printMLText("already_subscribed");
+				break;
+			case -4:
+				printMLText("internal_error");
+				break;
+			case 0:
+				printMLText("op_finished");
+				break;
 		}
-		$document->addNotify($groupid, false);
 	}
+
 }
-printMLText("op_finished");
 
 UI::contentContainerEnd();
 UI::htmlEndPage();
