@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -28,10 +28,11 @@ include("../inc/inc.DBAccess.php");
 include("../inc/inc.FileUtils.php");
 include("../inc/inc.Language.php");
 include("../inc/inc.ClassUI.php");
+include("../inc/inc.ClassEmail.php");
 include("../inc/inc.Authentication.php");
 
 if (!isset($_GET["action"]) || (strcasecmp($_GET["action"], "delnotify") && strcasecmp($_GET["action"], "addnotify"))) {
-	UI::htmlStartPage(getMLText("document_title", array("documentname" => getMLText("invalid_action"))));
+	UI::htmlStartPage(getMLText("folder_title", array("foldername" => getMLText("invalid_action"))));
 	UI::globalNavigation();
 	UI::pageNavigation(getMLText("invalid_action"));
 	UI::htmlEndPage();
@@ -40,7 +41,7 @@ if (!isset($_GET["action"]) || (strcasecmp($_GET["action"], "delnotify") && strc
 $action = $_GET["action"];
 
 if (isset($_GET["userid"]) && (!is_numeric($_GET["userid"]) || $_GET["userid"]<-1)) {
-	UI::htmlStartPage(getMLText("document_title", array("documentname" => getMLText("unknown_user"))));
+	UI::htmlStartPage(getMLText("folder_title", array("foldername" => getMLText("unknown_user"))));
 	UI::globalNavigation();
 	UI::pageNavigation(getMLText("unknown_user"));
 	UI::htmlEndPage();
@@ -49,7 +50,7 @@ if (isset($_GET["userid"]) && (!is_numeric($_GET["userid"]) || $_GET["userid"]<-
 $userid = $_GET["userid"];
 
 if (isset($_GET["groupid"]) && (!is_numeric($_GET["groupid"]) || $_GET["groupid"]<-1)) {
-	UI::htmlStartPage(getMLText("document_title", array("documentname" => getMLText("unknown_group"))));
+	UI::htmlStartPage(getMLText("folder_title", array("foldername" => getMLText("unknown_group"))));
 	UI::globalNavigation();
 	UI::pageNavigation(getMLText("unknown_group"));
 	UI::htmlEndPage();
@@ -79,7 +80,7 @@ UI::htmlStartPage(getMLText("folder_title", array("foldername" => $folder->getNa
 UI::globalNavigation($folder);
 UI::pageNavigation($folderPathHTML, "view_folder", $folder);
 
-if ($folder->getAccessMode($user) < M_READWRITE) {
+if ($folder->getAccessMode($user) < M_READ) {
 	UI::contentContainer(getMLText("operation_not_allowed"));
 	UI::htmlEndPage();
 	exit;
@@ -89,26 +90,30 @@ UI::contentHeading(getMLText("edit_folder_notify"));
 UI::contentContainerStart();
 
 //Benachrichtigung lschen ------------------------------------------------------------------------
-if ($action == "delnotify")
-{
+if ($action == "delnotify") {
 	printMLText("deleting_folder_notify");
 	if (isset($userid)) {
-		if (!is_object(getUser($userid))) {
-			printMLText("unknown_user");
-			UI::contentContainerEnd();
-			UI::htmlEndPage();
-			exit;
-		}
-		$folder->removeNotify($userid, true);
+		$res = $folder->removeNotify($userid, true);
 	}
 	else if (isset($groupid)) {
-		if (!is_object(getGroup($groupid))) {
-			printMLText("unknown_group");
-			UI::contentContainerEnd();
-			UI::htmlEndPage();
-			exit;
-		}
-		$folder->removeNotify($groupid, false);
+		$res = $folder->removeNotify($groupid, false);
+	}
+	switch ($res) {
+		case -1:
+			print (isset($userid) ? getMLText("unknown_user") : getMLText("unknown_group"));
+			break;
+		case -2:
+			printMLText("access_denied");
+			break;
+		case -3:
+			printMLText("already_subscribed");
+			break;
+		case -4:
+			printMLText("internal_error");
+			break;
+		case 0:
+			printMLText("op_finished");
+			break;
 	}
 }
 
@@ -116,27 +121,47 @@ if ($action == "delnotify")
 else if ($action == "addnotify") {
 	printMLText("adding_folder_notify");
 	if ($userid != -1) {
-		if (!is_object(getUser($userid))) {
-			printMLText("unknown_user");
-			UI::contentContainerEnd();
-			UI::htmlEndPage();
-			exit;
+		$res = $folder->addNotify($userid, true);
+		switch ($res) {
+			case -1:
+				printMLText("unknown_user");
+				break;
+			case -2:
+				printMLText("access_denied");
+				break;
+			case -3:
+				printMLText("already_subscribed");
+				break;
+			case -4:
+				printMLText("internal_error");
+				break;
+			case 0:
+				printMLText("op_finished");
+				break;
 		}
-		$folder->addNotify($userid, true);
 	}
 	if ($groupid != -1) {
-		if (!is_object(getGroup($groupid))) {
-			printMLText("unknown_group");
-			UI::contentContainerEnd();
-			UI::htmlEndPage();
-			exit;
+		$res = $folder->addNotify($groupid, false);
+		switch ($res) {
+			case -1:
+				printMLText("unknown_group");
+				break;
+			case -2:
+				printMLText("access_denied");
+				break;
+			case -3:
+				printMLText("already_subscribed");
+				break;
+			case -4:
+				printMLText("internal_error");
+				break;
+			case 0:
+				printMLText("op_finished");
+				break;
 		}
-		$folder->addNotify($groupid, false);
 	}
 }
 	
-printMLText("op_finished");
-
 UI::contentContainerEnd();
 UI::htmlEndPage();
 ?>
