@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -68,10 +68,18 @@ if ((!isset($pwd) || strlen($pwd)==0) && ($login != $guestUser->getLogin()))  {
 //
 
 $user = false;
-if (isset($settings->_ldapHost) && strlen($settings->_ldapHost)>0 &&
-		isset($settings->_ldapPort) && is_int($settings->_ldapPort)) {
-	$ds = ldap_connect($settings->_ldapHost, $settings->_ldapPort);
+if (isset($settings->_ldapHost) && strlen($settings->_ldapHost)>0) {
+	if (isset($settings->_ldapPort) && is_int($settings->_ldapPort)) {
+		$ds = ldap_connect($settings->_ldapHost, $settings->_ldapPort);
+	}
+	else {
+		$ds = ldap_connect($settings->_ldapHost);
+	}
 	if (!is_bool($ds)) {
+		// Ensure that the LDAP connection is set to use version 3 protocol.
+		// Required for most authentication methods, including SASL.
+		ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, 3);
+
 		// try an anonymous bind first. If it succeeds, get the DN for the user.
 		$bind = @ldap_bind($ds);
 		$dn = false;
@@ -217,7 +225,7 @@ else if (isset($_GET["referuri"]) && strlen($_GET["referuri"])>0) {
 	$referuri = urldecode($_GET["referuri"]);
 }
 if (isset($referuri) && strlen($referuri)>0) {
-	header("Location: http".(isset($_SERVER['HTTPS']) ? "s" : "")."://".$_SERVER['HTTP_HOST'] . $referuri);
+	header("Location: http".((isset($_SERVER['HTTPS']) && (strcmp($_SERVER['HTTPS'],'off')!=0)) ? "s" : "")."://".$_SERVER['HTTP_HOST'] . $referuri);
 }
 else {
 	header("Location: ../".(isset($settings->_siteDefaultPage) && strlen($settings->_siteDefaultPage)>0 ? $settings->_siteDefaultPage : "out/out.ViewFolder.php?folderid=1"));
