@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -251,6 +251,25 @@ if (isset($_GET["lastupdate"]) && $_GET["lastupdate"]!=null) {
 }
 */
 
+//
+// Get the page number to display. If the result set contains more than
+// 25 entries, it is displayed across multiple pages.
+//
+// This requires that a page number variable be used to track which page the
+// user is interested in, and an extra clause on the select statement.
+//
+// Default page to display is always one.
+$pageNumber=1;
+if (isset($_GET["pg"])) {
+	if (is_numeric($_GET["pg"]) && $_GET["pg"]>0) {
+		$pageNumber = (integer)$_GET["pg"];
+	}
+	else if (!strcasecmp($_GET["pg"], "all")) {
+		$pageNumber = "all";
+	}
+}
+
+
 // ------------------------------------- Suche starten --------------------------------------------
 $startTime = getTime();
 
@@ -264,10 +283,11 @@ if (!$db->createTemporaryTable("ttcontentid") || !$db->createTemporaryTable("tts
 	exit;
 }
 
-$searchQuery = "SELECT `tblDocuments`.*, ".
-	"`tblDocumentContent`.`version`, ".
-	"`tblDocumentStatusLog`.`status`, `tblDocumentLocks`.`userID` as `lockUser` ".
-	"FROM `tblDocumentContent` ".
+//$searchQuery = "SELECT `tblDocuments`.*, ".
+//	"`tblDocumentContent`.`version`, ".
+// "`tblDocumentStatusLog`.`status`, `tblDocumentLocks`.`userID` as `lockUser` ".
+
+$searchQuery = "FROM `tblDocumentContent` ".
 	"LEFT JOIN `tblDocuments` ON `tblDocuments`.`id` = `tblDocumentContent`.`document` ".
 	"LEFT JOIN `tblDocumentStatus` ON `tblDocumentStatus`.`documentID` = `tblDocumentContent`.`document` ".
 	"LEFT JOIN `tblDocumentStatusLog` ON `tblDocumentStatusLog`.`statusID` = `tblDocumentStatus`.`statusID` ".
@@ -299,7 +319,37 @@ else if (isset($_GET["pendingApproval"])) {
 	$searchQuery .= " AND `tblDocumentStatusLog`.`status` = 1";
 }
 
-// Send the search query to the database.
+// Count the number of rows that the search will produce.
+$resArr = $db->getResultArray("SELECT COUNT(*) AS num ".$searchQuery);
+$totalDocs = 0;
+if (is_numeric($resArr[0]["num"]) && $resArr[0]["num"]>0) {
+	$totalDocs = (integer)$resArr[0]["num"];
+}
+$totalPages = (integer)($totalDocs/25);
+if (($totalDocs%25) > 0) {
+	$totalPages++;
+}
+if (is_numeric($pageNumber) && $pageNumber>$totalPages) {
+	$pageNumber = $totalPages;
+}
+
+// If there are no results from the count query, then there is no real need
+// to run the full query. TODO: re-structure code to by-pass additional
+// queries when no initial results are found. In the meantime, make sure that
+// the page number is at least 1.
+if (is_numeric($pageNumber) && $pageNumber==0) {
+	$pageNumber = 1;
+}
+// Prepare the complete search query, including the LIMIT clause.
+$searchQuery = "SELECT `tblDocuments`.*, ".
+	"`tblDocumentContent`.`version`, ".
+	"`tblDocumentStatusLog`.`status`, `tblDocumentLocks`.`userID` as `lockUser` ".$searchQuery;
+
+if (is_numeric($pageNumber)) {
+	$searchQuery .= " LIMIT ".(($pageNumber-1)*25).", 25";
+}
+
+// Send the complete search query to the database.
 $resArr = $db->getResultArray($searchQuery);
 
 $searchTime = getTime() - $startTime;
@@ -308,16 +358,16 @@ $searchTime = round($searchTime, 2);
 
 UI::contentContainerStart();
 ?>
-<table width="100%">
+<table width="100%" style="border-collapse: collapse;">
 <tr>
-<td align="left">
+<td align="left" style="padding:0; margin:0;">
 <?php
 $numResults = count($resArr);
 if ($numResults == 0) {
 	printMLText("search_no_results", array("query" => $query, "pending_review" => (isset($_GET["pendingReview"]) ? ", ".getMLText("search_result_pending_review") : ""), "pending_approval" => (isset($_GET["pendingApproval"]) ? ", ".getMLText("search_result_pending_approval") : "")));
 }
 else {
-	printMLText("search_report", array("query" => $query, "count" => $numResults, "pending_review" => (isset($_GET["pendingReview"]) ? ", ".getMLText("search_result_pending_review") : ""), "pending_approval" => (isset($_GET["pendingApproval"]) ? ", ".getMLText("search_result_pending_approval") : "")));
+	printMLText("search_report", array("query" => $query, "count" => $totalDocs, "pending_review" => (isset($_GET["pendingReview"]) ? ", ".getMLText("search_result_pending_review") : ""), "pending_approval" => (isset($_GET["pendingApproval"]) ? ", ".getMLText("search_result_pending_approval") : "")));
 }
 ?>
 </td>
@@ -332,6 +382,8 @@ if ($numResults == 0) {
 	exit;
 }
 
+// UI::pageList($pageNumber, $totalPages, "../op/op.Search.php", $_GET);
+
 print "<table class=\"folderView\">";
 print "<thead>\n<tr>\n";
 print "<th></th>\n";
@@ -388,10 +440,12 @@ foreach ($resArr as $docArr) {
 		else {
 			$queryStr="SELECT `tblDocumentReviewers`.*, `tblDocumentReviewLog`.`status`, ".
 				"`tblDocumentReviewLog`.`comment`, `tblDocumentReviewLog`.`date`, ".
-				"`tblDocumentReviewLog`.`userID` ".
+				"`tblDocumentReviewLog`.`userID`, `tblUsers`.`fullName`, `tblGroups`.`name` AS `groupName` ".
 				"FROM `tblDocumentReviewers` ".
 				"LEFT JOIN `tblDocumentReviewLog` USING (`reviewID`) ".
 				"LEFT JOIN `ttreviewid` on `ttreviewid`.`maxLogID` = `tblDocumentReviewLog`.`reviewLogID` ".
+				"LEFT JOIN `tblUsers` on `tblUsers`.`id` = `tblDocumentReviewers`.`required`".
+				"LEFT JOIN `tblGroups` on `tblGroups`.`id` = `tblDocumentReviewers`.`required`".
 				"WHERE `ttreviewid`.`maxLogID`=`tblDocumentReviewLog`.`reviewLogID` ".
 				"AND `tblDocumentReviewers`.`documentID` = '". $docArr["id"] ."' ".
 				"AND `tblDocumentReviewers`.`version` = '". $docArr["version"] ."' ";
@@ -405,21 +459,19 @@ foreach ($resArr as $docArr) {
 						$required = null;
 						switch ($r["type"]) {
 							case 0: // Reviewer is an individual.
-								$required = getUser($r["required"]);
-								if (!is_object($required)) {
+								if (strlen($r["fullName"])==0) {
 									$reqName = "Unknown user id '".$r["required"]."'";
 								}
 								else {
-									$reqName = "[I]".$required->getFullName();
+									$reqName = "[I]".$r["fullName"];
 								}
 								break;
 							case 1: // Reviewer is a group.
-								$required = getGroup($r["required"]);
-								if (!is_object($required)) {
+								if (strlen($r["groupName"])==0) {
 									$reqName = "Unknown group id '".$r["required"]."'";
 								}
 								else {
-									$reqName = "[G]".$required->getName();
+									$reqName = "[G]".$r["groupName"];
 								}
 								break;
 						}
@@ -442,10 +494,12 @@ foreach ($resArr as $docArr) {
 		else {
 			$queryStr="SELECT `tblDocumentApprovers`.*, `tblDocumentApproveLog`.`status`, ".
 				"`tblDocumentApproveLog`.`comment`, `tblDocumentApproveLog`.`date`, ".
-				"`tblDocumentApproveLog`.`userID` ".
+				"`tblDocumentApproveLog`.`userID`, `tblUsers`.`fullName`, `tblGroups`.`name` AS `groupName` ".
 				"FROM `tblDocumentApprovers` ".
 				"LEFT JOIN `tblDocumentApproveLog` USING (`approveID`) ".
 				"LEFT JOIN `ttapproveid` on `ttapproveid`.`maxLogID` = `tblDocumentApproveLog`.`approveLogID` ".
+				"LEFT JOIN `tblUsers` on `tblUsers`.`id` = `tblDocumentApprovers`.`required`".
+				"LEFT JOIN `tblGroups` on `tblGroups`.`id` = `tblDocumentApprovers`.`required`".
 				"WHERE `ttapproveid`.`maxLogID`=`tblDocumentApproveLog`.`approveLogID` ".
 				"AND `tblDocumentApprovers`.`documentID` = '". $docArr["id"] ."' ".
 				"AND `tblDocumentApprovers`.`version` = '". $docArr["version"] ."'";
@@ -459,21 +513,19 @@ foreach ($resArr as $docArr) {
 						$required = null;
 						switch ($a["type"]) {
 							case 0: // Approver is an individual.
-								$required = getUser($a["required"]);
-								if (!is_object($required)) {
-									$reqName = "Unknown user id '".$r["required"]."'";
+								if (strlen($a["fullName"])==0) {
+									$reqName = "Unknown user id '".$a["required"]."'";
 								}
 								else {
-									$reqName = "[I]".$required->getFullName();
+									$reqName = "[I]".$a["fullName"];
 								}
 								break;
 							case 1: // Approver is a group.
-								$required = getGroup($a["required"]);
-								if (!is_object($required)) {
-									$reqName = "Unknown group id '".$r["required"]."'";
+								if (strlen($a["groupName"])==0) {
+									$reqName = "Unknown group id '".$a["required"]."'";
 								}
 								else {
-									$reqName = "[G]".$required->getName();
+									$reqName = "[G]".$a["groupName"];
 								}
 								break;
 						}
@@ -496,6 +548,9 @@ if ($resultsFilteredByAccess) {
 	print "<tr><td colspan=\"7\">". getMLText("search_results_access_filtered") . "</td></tr>";
 }
 print "</tbody></table>\n";
+
+UI::pageList($pageNumber, $totalPages, "../op/op.Search.php", $_GET);
+
 UI::contentContainerEnd();
 UI::htmlEndPage();
 ?>
