@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -186,8 +186,8 @@ if (is_uploaded_file($_FILES["userfile"]["tmp_name"]) && $_FILES["userfile"]["si
 			// Groups:
 			$grpReviewers = $contentResult->getReviewers("g");
 			if (count($grpReviewers)) {
-				UI::contentSubHeading(getMLText("individual_reviewers"));
-				print "<ul class=\"reviewers\">";
+				UI::contentSubHeading(getMLText("group_reviewers"));
+				print "<ul>";
 				if (count($grpReviewers[0])) {
 					foreach ($grpReviewers[0] as $reviewer) {
 						print "<li>".getMLText("group")." '".$reviewer->getName()."' ".getMLText("reviewer_added").". ";
