@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -24,11 +24,11 @@ include("../inc/inc.ClassDocument.php");
 include("../inc/inc.ClassFolder.php");
 include("../inc/inc.ClassGroup.php");
 include("../inc/inc.ClassUser.php");
+include("../inc/inc.ClassUI.php");
 include("../inc/inc.DBAccess.php");
 include("../inc/inc.FileUtils.php");
 include("../inc/inc.Language.php");
 include("../inc/inc.OutUtils.php");
-
 include("../inc/inc.Authentication.php");
 
 $folderid = sanitizeString($_GET["folderid"]);
@@ -46,75 +46,62 @@ function printTree($path, $level = 0)
 		$nextFolderID = $path[$level+1]->getID();
 	else
 		$nextFolderID = -1;
-	
-	print "<table cellpadding=0 cellspacing=0>\n";
-	print "  <tr>\n";
-	print "    <td valign=\"top\"";
-	if (count($subFolders) + count($documents) > 0)
-		print " background=\"".getImgPath("down.gif")."\"";
-	print "><img src=\"";
-	if ($level == 0) printImgPath("to_down.gif");
-	else if (count($subFolders) + count($documents) > 0) printImgPath("right_in_to_down.gif");
-	else printImgPath("right_in.gif");
-	print "\" border=0></td>\n";
-	print "    <td class=\"foldertree\">";
-	print "<img src=\"".getImgPath("folder_opened.gif")."\" width=18 height=18 border=0>".$folder->getName()."</td>\n";
-	print "  </tr>\n";
-	
+
+	if ($level == 0) {
+		print "<ul style='list-style-type: none;'>\n";
+	}
+	print "  <li>\n";
+	print "<img class='treeicon' src=\"";
+	if ($level == 0) UI::printImgPath("minus.png");
+	else if (count($subFolders) + count($documents) > 0) UI::printImgPath("minus.png");
+	else UI::printImgPath("blank.png");
+	print "\" border=0>\n";
+	if ($folder->getAccessMode($user) >= $accessMode) {
+		print "<a class=\"foldertree_selectable\" href=\"javascript:folderSelected(" . $folder->getID() . ", '" . sanitizeString($folder->getName()) . "')\">";
+		print "<img src=\"".UI::getImgPath("folder_opened.gif")."\" border=0>".$folder->getName()."</a>\n";
+	}
+	else
+		print "<img src=\"".UI::getImgPath("folder_opened.gif")."\" width=18 height=18 border=0>".$folder->getName()."\n";
+	print "  </li>\n";
+
+	print "<ul style='list-style-type: none;'>";
+
 	for ($i = 0; $i < count($subFolders); $i++) {
-		print "<tr>";
-		if ($i +1 < count($subFolders) + count($documents))
-			print "<td background=\"".getImgPath("down.gif")."\" valign=\"top\"><img src=\"".getImgPath("right.gif")."\" border=0></td>";
-		else
-			print "<td valign=\"top\"><img src=\"".getImgPath("right_last.gif")."\" border=0></td>";
-		print "<td>";
 		if ($subFolders[$i]->getID() == $nextFolderID)
 			printTree($path, $level+1);
 		else {
+			print "<li>\n";
 			$subFolders_ = filterAccess($subFolders[$i]->getSubFolders(), $user, M_READ);
 			$documents_  = filterAccess($subFolders[$i]->getDocuments(), $user, M_READ);
 			
-			print "<table cellpadding=0 cellspacing=0><tr>";
 			if (count($subFolders_) + count($documents_) > 0)
-				print "  <td valign=\"top\"><a href=\"out.DocumentChooser.php?form=$form&folderid=".$subFolders[$i]->getID()."\"><img src=\"".getImgPath("right_in_plus.gif")."\" border=0></a></td>";
+				print "<a href=\"out.DocumentChooser.php?form=$form&folderid=".$subFolders[$i]->getID()."\"><img class='treeicon' src=\"".getImgPath("plus.png")."\" border=0></a>";
 			else
-				print "  <td valign=\"top\"><img src=\"".getImgPath("right_in.gif")."\"></td>";
-			print "  <td class=\"foldertree\" valign=\"top\">";
-			print "<img src=\"".getImgPath("folder_closed.gif")."\" border=0>".$subFolders[$i]->getName()."</td>";
-			print "</tr></table>";
+				print "<img class='treeicon' src=\"".getImgPath("blank.png")."\">";
+			print "<img src=\"".getImgPath("folder_closed.gif")."\" border=0>".$subFolders[$i]->getName()."\n";
+			print "</li>";
 		}
-		print "</td>";
-		print "</tr>";
 	}
-	for ($i = 0; $i < count($documents); $i++)
-	{
-		print "<tr>";
-		if ($i +1 < count($documents))
-			print "<td background=\"images/down.gif\" valign=\"top\"><img src=\"images/right.gif\" border=0></td>";
-		else
-			print "<td valign=\"top\"><img src=\"images/right_last.gif\" border=0></td>";
-		print "<td>";
-		print "  <table cellpadding=0 cellspacing=0><tr>";
-		print "    <td valign=\"top\"><img src=\"images/right_in.gif\"></td>";
-		print "    <td><a  class=\"foldertree_selectable\" href=\"javascript:documentSelected(".$documents[$i]->getID().",'".sanitizeString($documents[$i]->getName())."');\"><img src=\"images/file.gif\" border=0>".$documents[$i]->getName()."</a></td>";
-		print "  </tr></table>";
-		print "</td>";
-		print "</tr>";
+	for ($i = 0; $i < count($documents); $i++) {
+		print "<li>\n";
+		print "<img class='treeicon' src=\"images/blank.png\">";
+		print "<a  class=\"foldertree_selectable\" href=\"javascript:documentSelected(".$documents[$i]->getID().",'".sanitizeString($documents[$i]->getName())."');\"><img src=\"images/file.gif\" border=0>".$documents[$i]->getName()."</a>";
+		print "</li>";
+	}
+
+	print "</ul>\n";
+	if ($level == 0) {
+		print "</ul>\n";
 	}
 	
-	print "</table>\n";
 }
 
 
-
+UI::htmlStartPage(getMLText("choose_target_document"));
+UI::globalBanner();
+UI::pageNavigation(getMLText("choose_target_document"));
 ?>
 
-
-<html>
-<head>
-<link rel="STYLESHEET" type="text/css" href="styles.css">
-<title><?php echo getMLText("choose_target_document")?></title>
-
 <script language="JavaScript">
 function decodeString(s) {
 	s = new String(s);
@@ -145,12 +132,11 @@ function documentSelected(id, name) {
 }
 </script>
 
-</head>
-<body>
-
 <?php
 	$folder = getFolder($folderid);
+	UI::contentContainerStart();
 	printTree($folder->getPath());
+	UI::contentContainerEnd();
 ?>
 
 <script language="JavaScript">
@@ -158,5 +144,6 @@ targetName = opener.document.<?php echo $form?>.docname;
 targetID   = opener.document.<?php echo $form?>.docid;
 </script>
 
-</body>
-</html>
+<?php
+UI::htmlEndPage();
+?>
