@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -53,12 +53,13 @@ UI::htmlStartPage(getMLText("document_title", array("documentname" => $document-
 UI::globalNavigation($folder);
 UI::pageNavigation($docPathHTML, "view_document");
 
-if ($document->getAccessMode($user) < M_READWRITE) {
+if ($document->getAccessMode($user) < M_READ) {
 	UI::contentHeading(getMLText("edit_existing_notify") . ": " . $document->getName());
 	UI::contentContainer(getMLText("access_denied"));
 	UI::htmlEndPage();
 	exit;
 }
+
 $notifyList = $document->getNotifyList();
 ?>
 <script language="JavaScript">
@@ -82,6 +83,9 @@ function checkForm()
 UI::contentHeading(getMLText("edit_existing_notify"));
 UI::contentContainerStart();
 
+$userNotifyIDs = array();
+$groupNotifyIDs = array();
+
 print "<table>\n";
 if ((count($notifyList["users"]) == 0) && (count($notifyList["groups"]) == 0)) {
 	print "<tr><td>".getMLText("empty_notify_list")."</td></tr>";
@@ -91,15 +95,21 @@ else {
 		print "<tr>";
 		print "<td><img src=\"images/usericon.gif\" class=\"mimeicon\"></td>";
 		print "<td>" . $userNotify->getFullName() . "</td>";
-		print "<td><a href=\"../op/op.DocumentNotify.php?documentid=". $documentid . "&action=delnotify&userid=".$userNotify->getID()."\">".getMLText("delete")."</a>";
+		if ($user->isAdmin() || $user->getID() == $userNotify->getID()) {
+			print "<td><a href=\"../op/op.DocumentNotify.php?documentid=". $documentid . "&action=delnotify&userid=".$userNotify->getID()."\">".getMLText("delete")."</a></td>";
+		}
 		print "</tr>";
+		$userNotifyIDs[] = $userNotify->getID();
 	}
 	foreach ($notifyList["groups"] as $groupNotify) {
 		print "<tr>";
 		print "<td><img src=\"images/groupicon.gif\" width=16 height=16 border=0></td>";
 		print "<td>" . $groupNotify->getName() . "</td>";
-		print "<td><a href=\"../op/op.DocumentNotify.php?documentid=". $documentid . "&action=delnotify&groupid=".$groupNotify->getID()."\">".getMLText("delete")."</a>";
+		if ($user->isAdmin() || $groupNotify->isMember($user)) {
+			print "<td><a href=\"../op/op.DocumentNotify.php?documentid=". $documentid . "&action=delnotify&groupid=".$groupNotify->getID()."\">".getMLText("delete")."</a></td>";
+		}
 		print "</tr>";
+		$groupNotifyIDs[] = $groupNotify->getID();
 	}
 }
 print "</table>\n";
@@ -118,11 +128,15 @@ UI::contentContainerStart();
 				<option value="-1"><?php printMLText("select_one");?>
 				<option value="-1">-------------------------------
 				<?php
-					$allUsers = getAllUsers();
-					foreach ($allUsers as $userObj) {
-						if ($userObj->getID() == $settings->_guestID)
-							continue;
-						print "<option value=\"".$userObj->getID()."\">" . $userObj->getFullName() . "\n";
+					if ($user->isAdmin()) {
+						$allUsers = getAllUsers();
+						foreach ($allUsers as $userObj) {
+							if ($userObj->getID() != $settings->_guestID && !in_array($userObj->getID(), $userNotifyIDs))
+								print "<option value=\"".$userObj->getID()."\">" . $userObj->getFullName() . "\n";
+						}
+					}
+					else if ($user->getID() != $settings->_guestID && !in_array($user->getID(), $userNotifyIDs)) {
+						print "<option value=\"".$user->getID()."\">" . $user->getFullName() . "\n";
 					}
 				?>
 			</select>
@@ -136,8 +150,11 @@ UI::contentContainerStart();
 				<option value="-1">-------------------------------
 				<?php
 					$allGroups = getAllGroups();
-					foreach ($allGroups as $groupObj)
-						print "<option value=\"".$groupObj->getID()."\">" . $groupObj->getName() . "\n";
+					foreach ($allGroups as $groupObj) {
+						if (($user->isAdmin() || $groupObj->isMember($user)) && !in_array($groupObj->getID(), $groupNotifyIDs)) {
+							print "<option value=\"".$groupObj->getID()."\">" . $groupObj->getName() . "\n";
+						}
+					}
 				?>
 			</select>
 		</td>
