@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -64,7 +64,13 @@ function checkForm()
 {
 	msg = "";
 	if (document.form1.name.value == "") msg += "<?php printMLText("js_no_name");?>\n";
+<?php
+	if (isset($settings->_strictFormCheck) && $settings->_strictFormCheck) {
+	?>
 	if (document.form1.comment.value == "") msg += "<?php printMLText("js_no_comment");?>\n";
+<?php
+	}
+?>
 	if (msg != "")
 	{
 		alert(msg);
