@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -52,7 +52,7 @@ UI::htmlStartPage(getMLText("folder_title", array("foldername" => $folder->getNa
 UI::globalNavigation($folder);
 UI::pageNavigation($folderPathHTML, "view_folder", $folder);
 
-if ($folder->getAccessMode($user) < M_READWRITE) {
+if ($folder->getAccessMode($user) < M_READ) {
 	UI::contentContainer(getMLText("access_denied"));
 	UI::htmlEndPage();
 	exit;
@@ -82,6 +82,9 @@ function checkForm()
 UI::contentHeading(getMLText("edit_existing_notify"));
 UI::contentContainerStart();
 
+$userNotifyIDs = array();
+$groupNotifyIDs = array();
+
 print "<table>\n";
 if ((count($notifyList["users"]) == 0) && (count($notifyList["groups"]) == 0)) {
 	print "<tr><td>".getMLText("empty_notify_list")."</td></tr>";
@@ -91,16 +94,22 @@ else {
 		print "<tr>";
 		print "<td><img src=\"images/usericon.gif\" class=\"mimeicon\"></td>";
 		print "<td>" . $userNotify->getFullName() . "</td>";
-		print "<td><a href=\"../op/op.FolderNotify.php?folderid=". $folderid . "&action=delnotify&userid=".$userNotify->getID()."\">".getMLText("delete")."</a>";
+		if ($user->isAdmin() || $user->getID() == $userNotify->getID()) {
+			print "<td><a href=\"../op/op.FolderNotify.php?folderid=". $folderid . "&action=delnotify&userid=".$userNotify->getID()."\">".getMLText("delete")."</a>";
+		}
 		print "</tr>";
+		$userNotifyIDs[] = $userNotify->getID();
 	}
 
 	foreach ($notifyList["groups"] as $groupNotify) {
 		print "<tr>";
 		print "<td><img src=\"images/groupicon.gif\" class=\"mimeicon\"></td>";
 		print "<td>" . $groupNotify->getName() . "</td>";
-		print "<td><a href=\"../op/op.FolderNotify.php?folderid=". $folderid . "&action=delnotify&groupid=".$groupNotify->getID()."\">".getMLText("delete")."</a>";
+		if ($user->isAdmin() || $groupNotify->isMember($user)) {
+			print "<td><a href=\"../op/op.FolderNotify.php?folderid=". $folderid . "&action=delnotify&groupid=".$groupNotify->getID()."\">".getMLText("delete")."</a>";
+		}
 		print "</tr>";
+		$groupNotifyIDs[] = $groupNotify->getID();
 	}
 }
 print "</table>\n";
@@ -120,12 +129,15 @@ UI::contentContainerStart();
 				<option value="-1"><?php printMLText("select_one");?>
 				<option value="-1">-------------------------------
 				<?php
-					$allUsers = getAllUsers();
-					foreach ($allUsers as $userObj) {
-						if ($userObj->getID() == $settings->_guestID) {
-							continue;
+					if ($user->isAdmin()) {
+						$allUsers = getAllUsers();
+						foreach ($allUsers as $userObj) {
+							if ($userObj->getID() != $settings->_guestID && !in_array($userObj->getID(), $userNotifyIDs))
+								print "<option value=\"".$userObj->getID()."\">" . $userObj->getFullName() . "\n";
 						}
-						print "<option value=\"".$userObj->getID()."\">" . $userObj->getFullName() . "\n";
+					}
+					else if ($user->getID() != $settings->_guestID && !in_array($user->getID(), $userNotifyIDs)) {
+						print "<option value=\"".$user->getID()."\">" . $user->getFullName() . "\n";
 					}
 				?>
 			</select>
@@ -140,7 +152,9 @@ UI::contentContainerStart();
 				<?php
 					$allGroups = getAllGroups();
 					foreach ($allGroups as $groupObj) {
-						print "<option value=\"".$groupObj->getID()."\">" . $groupObj->getName() . "\n";
+						if (($user->isAdmin() || $groupObj->isMember($user)) && !in_array($groupObj->getID(), $groupNotifyIDs)) {
+							print "<option value=\"".$groupObj->getID()."\">" . $groupObj->getName() . "\n";
+						}
 					}
 				?>
 			</select>
