@@ -1,7 +1,7 @@
 <?php
 //    MyDMS. Document Management System
 //    Copyright (C) 2002-2005  Markus Westphal
-//    Copyright (C) 2006 Malcolm Cowe
+//    Copyright (C) 2006-2008 Malcolm Cowe
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -158,21 +158,19 @@ if (count($documents) > 0) {
 					$required = null;
 					switch ($r["type"]) {
 						case 0: // Reviewer is an individual.
-							$required = getUser($r["required"]);
-							if (!is_object($required)) {
+							if (strlen($r["fullName"])==0) {
 								$reqName = "Unknown user id '".$r["required"]."'";
 							}
 							else {
-								$reqName = "[I]".$required->getFullName();
+								$reqName = "[I]".$r["fullName"];
 							}
 							break;
 						case 1: // Reviewer is a group.
-							$required = getGroup($r["required"]);
-							if (!is_object($required)) {
+							if (strlen($r["groupName"])==0) {
 								$reqName = "Unknown group id '".$r["required"]."'";
 							}
 							else {
-								$reqName = "[G]".$required->getName();
+								$reqName = "[G]".$r["groupName"];
 							}
 							break;
 					}
@@ -197,21 +195,19 @@ if (count($documents) > 0) {
 					$required = null;
 					switch ($a["type"]) {
 						case 0: // Approver is an individual.
-							$required = getUser($a["required"]);
-							if (!is_object($required)) {
-								$reqName = "Unknown user id '".$r["required"]."'";
+							if (strlen($a["fullName"])==0) {
+								$reqName = "Unknown user id '".$a["required"]."'";
 							}
 							else {
-								$reqName = "[I]".$required->getFullName();
+								$reqName = "[I]".$a["fullName"];
 							}
 							break;
 						case 1: // Approver is a group.
-							$required = getGroup($a["required"]);
-							if (!is_object($required)) {
-								$reqName = "Unknown group id '".$r["required"]."'";
+							if (strlen($a["groupName"])==0) {
+								$reqName = "Unknown group id '".$a["required"]."'";
 							}
 							else {
-								$reqName = "[G]".$required->getName();
+								$reqName = "[G]".$a["groupName"];
 							}
 							break;
 					}
